/*************************************************************
 * i2cm_main.c
 *
 * I2C master main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_I2CM
#include <mux_api.h>
#include <i2cm_api.h>
#include <t16_api.h>

// T16 timer channel number for I2C master
#define I2CM_MAIN_T16_CH			T16_CH2

// I2C slave address
#define I2CM_MAIN_SLAVE_ADDRESS	0x01

// send data parameter
#define I2CM_MAIN_SENDDATA		"FROM MST"
#define I2CM_MAIN_RECVDATA_NUM	8

// I2C send data buffer state flag
#define	I2CM_MAIN_TXEMPTY		0				// send data buffer is empty
#define	I2CM_MAIN_TXFULL		1				// send data buffer has a data
volatile static unsigned char FlagTXEmpty;		// send data buffer empty flag

// I2C receive data buffer state flag
#define	I2CM_MAIN_RXEMPTY		0				// send data buffer is empty
#define	I2CM_MAIN_RXFULL		1				// send data buffer has a data
volatile static unsigned char FlagRXEmpty;		// send data buffer empty flag

// prototype
void i2cMasterMain(void);
static void startI2Cmaster(void);
static void stopI2Cmaster(void);
static unsigned char sendI2CmasterData(unsigned short address, unsigned char *buf, unsigned char len);
static unsigned char recvI2CmasterData(unsigned short address, unsigned char *buf, unsigned char len);
static void startT16ForI2CM(unsigned char chNo);
void i2cMasterIntHandler(void)__attribute__((interrupt_handler));


/*
 * I2C master sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void i2cMasterMain(void)
{
	unsigned short	addrSlave;									// I2C slave address
	unsigned char	i2cMSTSendData[] = I2CM_MAIN_SENDDATA;		// send data
	unsigned char	i2cMSTRecvData[I2CM_MAIN_RECVDATA_NUM+1];	// receive data
	unsigned char 	waitPressEnter[2];							// key input buffer

	printf("<<< I2C master demonstration start >>>\n");

	// start T16
	startT16ForI2CM(I2CM_MAIN_T16_CH);

	// start I2C master
	startI2Cmaster();

	// send data
	printf("Transmitted data : %s\n", i2cMSTSendData);

	addrSlave = I2CM_MAIN_SLAVE_ADDRESS;
	sendI2CmasterData(addrSlave, &i2cMSTSendData[0], (sizeof(i2cMSTSendData) - 1));

	printf("Please press enter key\n");
	fgets(&waitPressEnter[0], 2, stdin);

	// receive data
	printf("Received data : ");

	recvI2CmasterData(addrSlave, &i2cMSTRecvData[0], I2CM_MAIN_RECVDATA_NUM);

	i2cMSTRecvData[I2CM_MAIN_RECVDATA_NUM] = '\0';
	printf("%s\n", &i2cMSTRecvData[0]);

	// stop I2C master
	stopI2Cmaster();

	// stop T16
	T16_setTimerRun(I2CM_MAIN_T16_CH, T16_TIMER_STOP);

	// clear port
	MUX_setI2CMport(MUX_I2CM_0, MUX_OFF);

	printf("<<< I2C master demonstration finish >>>\n");
}

/*
 * start I2C master.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startI2Cmaster(void)
{
	I2CM_init(I2CM_NOISEREMOVE_ON);
	
	I2CM_initInt(I2CM_INT_LEVEL3);
}

/*
 * stop I2C master.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopI2Cmaster(void)
{
	I2CM_initInt(I2CM_INT_LEVEL0);
}

/*
 * start I2C master data send.
 * param	unsigned short address   I2C slave address
 * param	unsigned char  *buf      send data
 * param	unsigned char  len       send data length excepting slave address
 * return	unsigned char            actual send data length
 * history	:	2009/12/25
 */
static unsigned char sendI2CmasterData(unsigned short address, unsigned char *buf, unsigned char len)
{
	unsigned short	addrsSlave;		// slave address
	unsigned char	actual;			// actual send data length
	unsigned char	*sendData;		// send data

	addrsSlave = address;
	actual = 0U;
	sendData = buf;

	FlagTXEmpty = I2CM_MAIN_TXFULL;

	// enable TX buffer empty
	I2CM_controlInt(I2CM_RXINT_DISABLE | I2CM_TXINT_ENABLE);
	I2CM_setComEnable(I2CM_CTL_ENABLE);

	// set start condition
	I2CM_genCondition(I2CM_CONDITION_START);

	// send data and wait sending is completed
	I2CM_transmitSlaveAddress(I2CM_ADDRESS_7BIT, I2CM_DIR_TX, &addrsSlave);
	I2CM_setTransmitData((unsigned char)addrsSlave);		// data set to data register

	// wait empty interrupt
	while(FlagTXEmpty == I2CM_MAIN_TXFULL) {
		;
	}

	// wait busy state is cleared
	while(I2CM_checkTransmitBusy() == I2CM_RX_BUSY) {
		;
	}

	// data send
	if(I2CM_getSlaveResponse() == I2CM_SLAVE_ACK) {	// check response
		while(actual < len) {
			FlagTXEmpty = I2CM_MAIN_TXFULL;

			I2CM_setTransmitData(*sendData);		// data set to data register

			// wait empty interrupt
			while(FlagTXEmpty == I2CM_MAIN_TXFULL) {
				;
			}

			// wait busy state is cleared
			while(I2CM_checkTransmitBusy() == I2CM_TX_BUSY) {
				;
			}

			if (I2CM_getSlaveResponse() == I2CM_SLAVE_ACK) {
				// send successed(ack)
				actual++;
				sendData++;
			} else {
				// send failed(nak)
				break;
			}
		}
	}

	// set stop condition
	I2CM_genCondition(I2CM_CONDITION_STOP);

	// disable interrupt
	I2CM_setComEnable(I2CM_CTL_DISABLE);
	I2CM_controlInt(I2CM_RXINT_DISABLE | I2CM_TXINT_DISABLE);

	return actual;
}

/*
 * start I2C master data receive.
 * param	unsigned short address   I2C slave address
 * param	unsigned char  *buf      receive data buffler
 * param	unsigned char  len       received data length
 * return	unsigned char            actual receive data length
 * history	:	2009/12/25
 */
static unsigned char recvI2CmasterData(unsigned short address, unsigned char *buf, unsigned char len)
{
	unsigned short	addrsSlave;		// slave address
	unsigned char	actual;			// actual receive data length
	unsigned char	*recvData;		// receive data

	addrsSlave = address;
	actual = 0;
	recvData = buf;

	FlagTXEmpty = I2CM_MAIN_TXFULL;

	// enable TX buffer empty
	I2CM_controlInt(I2CM_RXINT_DISABLE | I2CM_TXINT_ENABLE);	// enable rx buffer full
	I2CM_setComEnable(I2CM_CTL_ENABLE);

	// set start condition
	I2CM_genCondition(I2CM_CONDITION_START);

	// send data and wait sending is completed
	I2CM_transmitSlaveAddress(I2CM_ADDRESS_7BIT, I2CM_DIR_RX, &addrsSlave);
	I2CM_setTransmitData((unsigned char)addrsSlave);		// data set to data register

	// wait empty interrupt
	while(FlagTXEmpty == I2CM_MAIN_TXFULL) {
		;
	}

	// wait busy state is cleared
	while(I2CM_checkTransmitBusy() == I2CM_TX_BUSY) {
		;
	}

	// data receive
	// check response
	while(I2CM_getSlaveResponse() != I2CM_SLAVE_ACK) {
		return actual;		// NAK
	}

	I2CM_setComEnable(I2CM_CTL_DISABLE);
	I2CM_controlInt(I2CM_RXINT_ENABLE | I2CM_TXINT_DISABLE);	// enable rx buffer full
	I2CM_setComEnable(I2CM_CTL_ENABLE);
	I2CM_getReceiveData(recvData);

	while(actual < len) {
		FlagRXEmpty = I2CM_MAIN_RXEMPTY;

		if(actual != (len - 1U)) {
			I2CM_setReceiveStart(I2CM_SLAVE_ACK);			// response ACK
		} else {
			I2CM_setReceiveStart(I2CM_SLAVE_NAK);			// response NAK
		}

		// wait receive interrupt
		while(FlagRXEmpty == I2CM_MAIN_RXEMPTY) {
			;
		}

		// data read from data register
		I2CM_getReceiveData(recvData);

		actual++;
		recvData++;
	}

	// set stop condition
	I2CM_genCondition(I2CM_CONDITION_STOP);

	// disable interrupt
	I2CM_setComEnable(I2CM_CTL_DISABLE);
	I2CM_controlInt(I2CM_RXINT_DISABLE | I2CM_TXINT_DISABLE);

	return actual;
}

/*
 * start T16 timer for I2C clock.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void startT16ForI2CM(unsigned char chNo)
{
	T16_TIM_MODE	mode;	// timer mode setting

	mode.ckactv	= T16_TIMMODE_ACTVL;
	mode.cksl	= T16_TIMMODE_INTERNAL;
	mode.trmd	= T16_TIMMODE_REPEAT;
	T16_init(chNo, T16_CNTCLK_16, &mode, 1);
	T16_setTimerRun(chNo, T16_TIMER_RUN);
}

/*
 * I2C master interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void i2cMasterIntHandler(void)
{
	// receive data ready
	if(I2CM_checkReceiveReg() == I2CM_RXREG_NOTEMPTY) {
		FlagRXEmpty = I2CM_MAIN_RXFULL;
	}

	// transmit buffer empty
	if(I2CM_checkTransmitReg() == I2CM_TXREG_EMPTY) {
		FlagTXEmpty = I2CM_MAIN_TXEMPTY;
	}
}
#endif	// end of PE_I2CM
