/*************************************************************
 * remc_rx_main.c
 *
 * REMC receive main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_REMC_RX
#include <remc_api.h>
#include <mux_api.h>

// carrier parameter
#define	REMCRX_MAIN_CARRIER_H			38U
#define	REMCRX_MAIN_CARRIER_L			38U

#define	REMCRX_MAIN_SENDDATA_NUM		10U	// receive data number
#define	RX_NUMBER						6								// TX/RX number

// REMC receive data status
#define	REMCRX_MAIN_STATUS_IDLE			0	// idle
#define	REMCRX_MAIN_STATUS_HIGH			1	// high state
#define	REMCRX_MAIN_STATUS_LOW			2	// low state
typedef struct tag_REMC_MAIN_RX_PARAM {
	unsigned char stateRx;					// send data state
	unsigned char *dataRx;					// receive data buffer
	unsigned char counterRx;				// receive data counter
	unsigned char dataNumber;				// receive data number
	unsigned short dataTotal;				// receive data Total
	unsigned short dataTemp;				// receive data temporary buffer
} REMC_MAIN_RX_PARAM;
volatile static REMC_MAIN_RX_PARAM paramRx;

// prototype
void remcRxMain(void);
static void startREMCRx(void);
static void stopREMCRx(void);
static unsigned char recvData(unsigned char *buf);
void remcRxUnderFlowIntrHandler(void);
void remcRxFallIntrHandler(void);
void remcRxRiseIntrHandler(void);

/*
 * REMC rx sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcRxMain(void)
{
	unsigned char	data[REMCRX_MAIN_SENDDATA_NUM];						// data buffer
	unsigned char	i;													// counter

	printf("<<< REMC receive demonstration start >>>\n");
	
	// start remc
	startREMCRx();

	// receive data
	recvData(&data[0]);

	// stop remc
	stopREMCRx();

	// receive data all display
	for(i = 0; i < REMCRX_MAIN_SENDDATA_NUM; i++) {
		printf("Received data : %02d\n", data[i]);
	}
	
	// clear port
	MUX_setREMCport(MUX_OFF);
	
	printf("<<< REMC receive demonstration finish >>>\n");
}

/*
 * start REMC for data receive.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startREMCRx(void)
{
	REMC_INIT_PARAM param;										// initialize parameter

	param.selectCarrier = REMC_CLK2;
	param.selectLength = REMC_CLK1024;
	param.carrireL = REMCRX_MAIN_CARRIER_L;
	param.carrireH = REMCRX_MAIN_CARRIER_H;
	REMC_init(&param);
	
	REMC_initInt(REMC_INT_LEVEL3);
	REMC_controlInt(REMC_UDFINT_ENABLE | REMC_RISINGINT_ENABLE | REMC_FALLINGINT_ENABLE);
	REMC_setEnable(REMC_RECEIVE, REMC_ENABLE);
}

/*
 * stop REMC for data send.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopREMCRx(void)
{
	REMC_setEnable(REMC_RECEIVE, REMC_DISABLE);
	REMC_controlInt(REMC_UDFINT_DISABLE | REMC_RISINGINT_DISABLE | REMC_FALLINGINT_DISABLE);
	REMC_initInt(REMC_INT_LEVEL0);
}

/*
 * recv data.
 * param	unsigned char  *buf      send data buffler
 * return	unsigned char            actual received data length
 * history	:	2009/12/25
 */
static unsigned char recvData(unsigned char *buf)
{
	paramRx.counterRx = 0;
	paramRx.dataTotal = 0;
	paramRx.dataNumber = 0;
	paramRx.dataRx = buf;
	paramRx.stateRx = REMCRX_MAIN_STATUS_IDLE;

	REMC_setReceiveLength();

	while((paramRx.stateRx != REMCRX_MAIN_STATUS_IDLE) ||
		(paramRx.dataNumber < REMCRX_MAIN_SENDDATA_NUM)) {
		;
	}

	paramRx.dataNumber = 0;
	paramRx.stateRx = REMCRX_MAIN_STATUS_IDLE;

	return paramRx.dataNumber;
}


/*
 * REMC under flow interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcRxUnderFlowIntrHandler(void)
{
	paramRx.counterRx = 0;												// counter clear
	paramRx.dataTotal = 0;												// data clear
	paramRx.stateRx = REMCRX_MAIN_STATUS_IDLE;							// next idle state
}

/*
 * REMC falling edge interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcRxFallIntrHandler(void)
{
	// check status
	if(paramRx.stateRx == REMCRX_MAIN_STATUS_HIGH) {					// high section
		REMC_calcReceiveDataPulse((unsigned short *)&paramRx.dataTemp);

		// receive data length add
		paramRx.dataTotal += paramRx.dataTemp;
		paramRx.counterRx++;
		
		paramRx.stateRx = REMCRX_MAIN_STATUS_LOW;						// next low section
		REMC_setReceiveLength();
	} else {
		paramRx.counterRx = 0;											// counter clear
		paramRx.dataTotal = 0;											// data clear
		paramRx.stateRx = REMCRX_MAIN_STATUS_IDLE;						// next idle state
	}
}

/*
 * REMC rising edge interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcRxRiseIntrHandler(void)
{
	// check status
	if(paramRx.stateRx == REMCRX_MAIN_STATUS_IDLE) {						// idle state
		REMC_setReceiveLength();
		paramRx.stateRx = REMCRX_MAIN_STATUS_HIGH;						// next high section
	} else if(paramRx.stateRx == REMCRX_MAIN_STATUS_LOW) {				// low section
		REMC_calcReceiveDataPulse((unsigned short *)&paramRx.dataTemp);
		REMC_setReceiveLength();
		
		// receive data length add
		paramRx.dataTotal += paramRx.dataTemp;
		paramRx.counterRx++;
		
		// one data finish ?
		if(paramRx.counterRx >= ((RX_NUMBER - 1) * 2)) {
			*paramRx.dataRx = (unsigned char) (paramRx.dataTotal / paramRx.counterRx);
			
			// data clear for next data
			paramRx.dataRx++;
			paramRx.dataNumber++;
			paramRx.dataTotal = 0;
			paramRx.counterRx = 0;
		}
		paramRx.stateRx = REMCRX_MAIN_STATUS_HIGH;						// next high section
	} else {
		paramRx.counterRx = 0;											// counter clear
		paramRx.dataTotal = 0;											// data clear
		paramRx.stateRx = REMCRX_MAIN_STATUS_IDLE;						// next idle state
	}
}


#endif	// end of PE_REMC_RX
