/**
 * @file	i2cs.c
 * @brief	I2C slave driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_I2CS
#include <i2cs_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */
#define	SLV_ADDRESS_MAX						0x007f							// slave address max


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
void I2CS_reset(void);
short I2CS_setAddress(unsigned char address);
short I2CS_setClockStretch(unsigned short selectBit);
short I2CS_setAsyncDetection(unsigned short selectBit);
short I2CS_setNoiseRemove(unsigned short selectBit);
short I2CS_setBusFreeReq(unsigned short selectBit);
short I2CS_setReceiveResponse(unsigned short selectBit);
short I2CS_init(unsigned char address, unsigned short noise, unsigned short stretch);
short I2CS_setEnable(unsigned short selectBit);
short I2CS_setComEnable(unsigned short selectBit);
void I2CS_setTransmitData(unsigned char data);
void I2CS_getReceiveData(unsigned char *data);
short I2CS_initInt(unsigned char intLevel);
short I2CS_controlInt(unsigned short selectBit);
short I2CS_resetIntFlag(unsigned short selectBit);
void I2CS_checkBusStatusIntFlag(unsigned short *flag);
void I2CS_checkIntFlag(unsigned short *flag);
void I2CS_checkAccessStatus(unsigned short *flag);


/**
 * @brief	I2C slave software reset.
 * - 1. I2C slave software reset.
 *
 * @return	none
 * @date	2009/12/25
 */
void I2CS_reset(void)
{
	// software reset
	RegModify(R16_I2CS_CTL, SOFTRESET, SOFTRESET_RESET);
	RegModify(R16_I2CS_CTL, SOFTRESET, SOFTRESET_CANCEL);
}

/**
 * @brief	set an I2C slave address.
 * - 1. check parameters.
 * - 2. set an I2C slave address.
 *
 * @param	address; unsigned char:			I2C slave address
 * -		from 0x00 to 0x7F
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setAddress(unsigned char address)
{
	// parameter check
	if(address > SLV_ADDRESS_MAX) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// I2C slave address set
	RegWrite(R16_I2CS_SADRS, (unsigned short)address);
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a clock stretch function.
 * - 1. check parameters.
 * - 2. set a clock stretch function.
 *
 * @param	selectBit; unsigned short:		select clock stretch function
 * -		I2CS_CLKSTRETCH_OFF : clock stretch function off
 * -		I2CS_CLKSTRETCH_ON  : clock stretch function on
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setClockStretch(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_CLKSTRETCH_ON) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// clock stretch set
	RegModify(R16_I2CS_CTL, CLKSTR_EN, (selectBit << CLKSTR_EN_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set an async. address detection function.
 * - 1. check parameters.
 * - 2. reset an async. address detection.(when select on async. address detection function)
 * - 3. enable bus status interrupt.(when select on async. address detection function)
 * - 4. async. address detection function on/off.
 *
 * @param	selectBit; unsigned short:		select async. address detection function
 * -		I2CS_ASYNCDETCT_OFF : async. address detection function off
 * -		I2CS_ASYNCDETCT_ON  : async. address detection function on
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setAsyncDetection(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_ASYNCDETCT_ON) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// async. address detection set
	if(selectBit == I2CS_ASYNCDETCT_ON) {
		RegModify(R16_I2CS_STAT, ASDET, I2CS_ASYNC_DETECTED);						// detection reset
		RegModify(R16_I2CS_ICTL, BSTAT_IEN, BSTAT_IEN);								// bus status interrupt enable
	}
	RegModify(R16_I2CS_CTL, ASDET_EN, (selectBit << ASDET_EN_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a noise removal function.
 * - 1. check parameters.
 * - 2. set a noise removal function.
 *
 * @param	selectBit; unsigned short:		select noise removal function
 * -		I2CS_NOISEREMOVE_OFF : noise removal function off
 * -		I2CS_NOISEREMOVE_ON  : noise removal function on
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setNoiseRemove(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_NOISEREMOVE_ON) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// noise filter set
	RegModify(R16_I2CS_CTL, NF_EN, (selectBit << NF_EN_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a bus free request enable / disable.
 * - 1. check parameters.
 * - 2. set a bus free request enable / disable.
 *
 * @param	selectBit; unsigned short:		select enable / disable
 * -		I2CS_BUSFREE_DISABLE : bus free request disable
 * -		I2CS_BUSFREE_ENABLE  : bus free request enable
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setBusFreeReq(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_BUSFREE_ENABLE) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// bus free request enable / disable
	RegModify(R16_I2CS_CTL, BFREQ_EN, (selectBit << BFREQ_EN_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a data receive response.
 * - 1. check parameters.
 * - 2. set a data receive response.
 *
 * @param	selectBit; unsigned short:		select response
 * -		I2CS_RESDATA_ACK : data receive response ACK
 * -		I2CS_RESDATA_NAK : data receive response NAK
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setReceiveResponse(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_RESDATA_NAK) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// data receive response set
	RegModify(R16_I2CS_CTL, NAK_ANS, (selectBit << NAK_ANS_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	I2C slave initialize.
 * - 1. check parameters.
 * - 2. disable I2C slave.
 * - 3. set I2C slave ports.
 * - 4. reset an I2C slave.
 * - 5. set an I2C slave address.
 * - 6. set a clock stretch function.
 * - 7. set a noise removal function.
 *
 * @param	address; unsigned char: 		select slave address
 * -		from 0x00 to 0x7F
 * @param	noise; unsigned short: 			select noise removal function
 * -		I2CS_NOISEREMOVE_OFF : noise removal function off
 * -		I2CS_NOISEREMOVE_ON  : noise removal function on
 * @param	stretch; unsigned short: 		select clock stretch function
 * -		I2CS_CLKSTRETCH_OFF : clock stretch function off
 * -		I2CS_CLKSTRETCH_ON  : clock stretch function on
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_init(unsigned char address, unsigned short noise, unsigned short stretch)
{
	// parameter check
	if(	(address > SLV_ADDRESS_MAX) ||
		(stretch > I2CS_CLKSTRETCH_ON) ||
		(noise > I2CS_NOISEREMOVE_ON) ) {
			return	I2CS_INVALID_PARAMETER;
	}
	
	// I2C slave disable
	I2CS_setEnable(I2CS_CTL_DISABLE);
	
	// set I2C slave port
	MUX_setI2CSport(MUX_ON);
	
	// I2C slave reset
	I2CS_reset();
	
	// slave address set
	I2CS_setAddress(address);
	
	// clock stretch set
	I2CS_setClockStretch(stretch);
	
	// noise filter set
	I2CS_setNoiseRemove(noise);
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set an I2C slave enable / disable.
 * - 1. check parameters.
 * - 2. set an I2C slave enable/disable.
 *
 * @param	selectBit; unsigned short:		I2C slave enable / disable
 * -		I2CS_CTL_DISABLE : I2C slave disable
 * -		I2CS_CTL_ENABLE  : I2C slave enable
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setEnable(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_CTL_ENABLE) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// I2C slave enable / disable
	RegModify(R16_I2CS_CTL, I2C_EN, (selectBit << I2C_EN_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a data transfer enable / disable.
 * - 1. check parameters.
 * - 2. set an I2C slave data transfer enable/disable.
 *
 * @param	selectBit; unsigned short:		data transfer enable / disable
 * -		I2CS_COM_DISABLE : I2C slave data transfer disable
 * -		I2CS_COM_ENABLE  : I2C slave data transfer enable
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_setComEnable(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_COM_ENABLE) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// data TX / RX enable / disable
	RegModify(R16_I2CS_CTL, COM_MODE, (selectBit << COM_MODE_SFT));
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	set a transmit data.
 * - 1. set a transmit data.
 *
 * @param	data; unsigned char:			transmit data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2009/12/25
 */
void I2CS_setTransmitData(unsigned char data)
{
	// transmit data set
	RegWrite(R16_I2CS_TRNS, (unsigned short)data);
}

/**
 * @brief	get a receive data.
 * - 1. get a receive data and set to "data".
 *
 * @param	*data; unsigned char:			receive data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2009/12/25
 */
void I2CS_getReceiveData(unsigned char *data)
{
	// receive data get
	*data = (unsigned char)RegRead(R16_I2CS_RECV);
}

/**
 * @brief	I2C slave interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		I2CS_INT_LEVEL0 : level0
 * -		I2CS_INT_LEVEL1 : level1
 * -		I2CS_INT_LEVEL2 : level2
 * -		I2CS_INT_LEVEL3 : level3
 * -		I2CS_INT_LEVEL4 : level4
 * -		I2CS_INT_LEVEL5 : level5
 * -		I2CS_INT_LEVEL6 : level6
 * -		I2CS_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= I2CS_INT_LEVELMAX) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(I2CS_INTLVL, intLevel);
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	I2C slave interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable a bus status interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		I2CS_BUSINT_DISABLE : bus status interrupt disable
 * -		I2CS_BUSINT_ENABLE  : bus status interrupt enable
 * -		I2CM_RXINT_DISABLE  : receive interrupt disable
 * -		I2CM_RXINT_ENABLE   : receive interrupt enable
 * -		I2CM_TXINT_DISABLE  : transmit interrupt disable
 * -		I2CM_TXINT_ENABLE   : transmit interrupt enable
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_controlInt(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CS_IEN_MASK) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	if((selectBit & I2CS_BUSINT_ENABLE) != 0) {								// bus status interrupt
		I2CS_resetIntFlag(I2CS_UDFOVF_RESET | I2CS_BUSFREE_RESET | I2CS_MISMATCH_RESET |
			I2CS_ASYNC_RESET | I2CS_RESPONSE_RESET | I2CS_STOP_RESET);
	}
	
	// set interrupt enable / disable
	RegWrite(R16_I2CS_ICTL, selectBit);
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	reset an I2C slave bus status interrupt flag.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	selectBit; unsigned short:		reset interrupt flag
 * -		I2CS_UDFOVF_RESET   : data underflow/overflow interrupt flag reset
 * -		I2CS_BUSFREE_RESET  : bus free request interrupt flag reset
 * -		I2CS_MISMATCH_RESET : output data mismatch interrupt flag reset
 * -		I2CS_ASYNC_RESET    : async. address detection interrupt flag reset
 * -		I2CS_RESPONSE_RESET : master response interrupt flag reset
 * -		I2CS_STOP_RESET     : stop condition detect interrupt flag reset
 * @return	short:							end status
 * -		I2CS_SUCCESS : success
 * -		I2CS_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CS_resetIntFlag(unsigned short selectBit)
{
	// parameter check
	if((selectBit & ~I2CS_BUSINT_MASK) != 0) {
		return	I2CS_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegModify(R16_I2CS_STAT, I2CS_BUSINT_MASK, selectBit);
	
	return	I2CS_SUCCESS;
}

/**
 * @brief	check an I2C slave bus status interrupt flag.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned short:			check bus status interrupt flag
 * -		I2CS_BUSSTS_UNCHANGED    : bus status transition unchanged
 * -		I2CS_BUSSTS_CHANGED      : bus status transition changed
 * -		I2CS_UDFOVF_NOTOCCURRED  : data underflow/overflow is not occurred
 * -		I2CS_UDFOVF_OCCURRED     : data underflow/overflow is occurred
 * -		I2CS_BUSFREE_NOTOCCURRED : bus free request is not occurred
 * -		I2CS_BUSFREE_OCCURRED    : bus free request is occurred
 * -		I2CS_MISMATCH_NORMAL     : output data mismatch normal
 * -		I2CS_MISMATCH_ERROR      : output data mismatch error
 * -		I2CS_ASYNC_NOTDETECTED   : async. address detection status is not detected
 * -		I2CS_ASYNC_DETECTED      : async. address detection status is detected
 * -		I2CS_RESPONSE_ACK        : master response ACK
 * -		I2CS_RESPONSE_NAK        : master response NAK
 * -		I2CS_STOP_NOTDETECTED    : stop condition is not detect
 * -		I2CS_STOP_DETECTED       : stop condition is detect
 * @return	none
 * @date	2009/12/25
 */
void I2CS_checkBusStatusIntFlag(unsigned short *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_I2CS_STAT);
}

/**
 * @brief	I2C slave interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned short:			check interrupt flag
 * -		I2CS_RXDATA_NOTREADY : receive data is not ready
 * -		I2CS_RXDATA_READY    : receive data is ready
 * -		I2CS_TXDATA_NOTEMPTY : transmit data is not empty
 * -		I2CS_TXDATA_EMPTY    : transmit data is empty
 * @return	none
 * @date	2009/12/25
 */
void I2CS_checkIntFlag(unsigned short *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_I2CS_ASTAT) & I2CS_INT_MASK;
}

/**
 * @brief	I2C slave access status check.
 * - 1. read an access status and set the status to "flag".
 * 
 * @param	*flag; unsigned short:			check access status
 * -		I2CS_BUSSTS_FREE       : bus status free
 * -		I2CS_BUSSTS_BUSY       : bus status busy
 * -		I2CS_SLAVE_NOTSELECTED : slave is not selected
 * -		I2CS_SLAVE_SELECTED    : slave is selected
 * -		I2CS_DIRECTION_INPUT   : read/write direction input(write by master)
 * -		I2CS_DIRECTION_OUTPUT  : read/write direction output(read by master)
 * @return	none
 * @date	2009/12/25
 */
void I2CS_checkAccessStatus(unsigned short *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_I2CS_ASTAT) & I2CS_STATUS_MASK;
}

#endif	// end of PE_I2CS
