/**
 * @file	misc.c
 * @brief	MISC driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_MISC
#include <misc_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short MISC_setFlashReadAccessCycle(unsigned short selectBit);
short MISC_setOSC1PeripheralControl(unsigned short selectBit);
void MISC_controlWriteProtect(unsigned short selectBit);
short MISC_setIRAMSize(unsigned short selectAddress, unsigned short selectSize);
short MISC_setTTBR(unsigned short ttbrL, unsigned short ttbrH);
void MISC_getPSR(unsigned short *data);


/**
 * @brief	set a Flash controler read access cycle.
 * - 1. check parameters.
 * - 2. set a Flash controler read access cycle.
 *
 * @param	selectBit; unsigned short: 		select access cycle
 * -		MISC_READ_CYCLE2 : 2-cycle of read access cycle 
 * -		MISC_READ_CYCLE3 : 3-cycle of read access cycle 
 * -		MISC_READ_CYCLE4 : 4-cycle of read access cycle 
 * -		MISC_READ_CYCLE5 : 5-cycle of read access cycle 
 * -		MISC_READ_CYCLE1 : 1-cycle of read access cycle 
 * @return	short:							end status
 * -		MISC_SUCCESS : success
 * -		MISC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MISC_setFlashReadAccessCycle(unsigned short selectBit)
{
	// parameter check
	if(selectBit >= MISC_READ_MAX) {
		return	MISC_INVALID_PARAMETER;
	}
	
	// Flash controler read access cycle set
	RegModify(R16_MISC_FL, FLCYC, (selectBit << FLCYC_SFT));
	
	return	MISC_SUCCESS;
}

/**
 * @brief	set a peripheral control in debug mode.
 * - 1. check parameters.
 * - 2. set an OSC1 peripheral control in debug mode.
 *
 * @param	selectBit; unsigned short: 		select peripheral control
 * -		MISC_OSC1PERIPHERAL_STOP : OSC1 peripheral stop
 * -		MISC_OSC1PERIPHERAL_RUN  : OSC1 peripheral run
 * @return	short:							end status
 * -		MISC_SUCCESS : success
 * -		MISC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MISC_setOSC1PeripheralControl(unsigned short selectBit)
{
	// parameter check
	if(selectBit > MISC_OSC1PERIPHERAL_RUN) {
		return	MISC_INVALID_PARAMETER;
	}
	
	// peripheral control in debug mode set
	RegWrite(R16_MISC_OSC1, selectBit);
	
	return	MISC_SUCCESS;
}

/**
 * @brief	control a MISC register write protect.
 * - 1. check parameters.
 * - 2. control a write protect for IRAM size/vector table address register.
 *
 * @param	selectBit; unsigned short: 		select write protect
 * -		MISC_WRITEPROTECT_ON  : write protect on
 * -		MISC_WRITEPROTECT_OFF : write protect off
 * @return	none
 * @date	2009/12/25
 */
void MISC_controlWriteProtect(unsigned short selectBit)
{
	// MISC register write protect control
	RegWrite(R16_MISC_PROT, selectBit);
}

/**
 * @brief	set an IRAM size.
 * - 1. check parameters.
 * - 2. write protect off.
 * - 3. set a debug base address.
 * - 4. set an IRAM size.
 * - 5. write protect on.
 *
 * @param	selectAddress; unsigned short: 	select debug base address
 * -		MISC_DEBUGADDRESS_F : debug base address 0xfffc00
 * -		MISC_DEBUGADDRESS_0 : debug base address 0x000000
 * @param	selectSize; unsigned short: 	select IRAM size
 * -		MISC_IRAMSIZE_12KB : IRAM size 12KB
 * -		MISC_IRAMSIZE_8KB  : IRAM size 8KB
 * -		MISC_IRAMSIZE_4KB  : IRAM size 4KB
 * -		MISC_IRAMSIZE_2KB  : IRAM size 2KB
 * @return	short:							end status
 * -		MISC_SUCCESS : success
 * -		MISC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MISC_setIRAMSize(unsigned short selectAddress, unsigned short selectSize)
{
	// parameter check
	if(	(selectAddress > MISC_DEBUGADDRESS_0) ||
		(selectSize >= MISC_IRAMSIZE_MAX) ) {
			return	MISC_INVALID_PARAMETER;
	}
#ifndef	S1C17602
	if(selectSize < MISC_IRAMSIZE_2KB) {
		return	MISC_INVALID_PARAMETER;
	}
#endif	// end of S1C17602
	
	// MISC register write protect off
	MISC_controlWriteProtect(MISC_WRITEPROTECT_OFF);
	
	// IRAM size & debug base address set
	RegWrite(R16_MISC_IRAMSZ, ((selectAddress << DBADR_SFT) | (selectSize << IRAMSZ_SFT)));
	
	// MISC register write protect on
	MISC_controlWriteProtect(MISC_WRITEPROTECT_ON);
	
	return	MISC_SUCCESS;
}

/**
 * @brief	set a vector table address.
 * - 1. check parameters.
 * - 2. write protect off.
 * - 3. set a vector table address A[15:8].
 * - 4. set a vector table address A[23:16].
 * - 5. write protect on.
 *
 * @param	ttbrL; unsigned short: 			vector table base address A[15: 8]
 * -		from 0x00 to 0xFF
 * @param	ttbrH; unsigned short: 			vector table base address A[23:16]
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		MISC_SUCCESS : success
 * -		MISC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MISC_setTTBR(unsigned short ttbrL, unsigned short ttbrH)
{
	// parameter check
	if(	(ttbrL > MISC_DATA_MAX) ||
		(ttbrH > MISC_DATA_MAX) ) {
			return	MISC_INVALID_PARAMETER;
	}
	
	// MISC register write protect off
	MISC_controlWriteProtect(MISC_WRITEPROTECT_OFF);
	
	// vector table address A[15:08] set
	RegWrite(R16_MISC_TTBRL, (ttbrL << TTBRL_SFT));
	
	// vector table address A[23:16] set
	RegWrite(R16_MISC_TTBRH, (ttbrH << TTBRH_SFT));
	
	// MISC register write protect on
	MISC_controlWriteProtect(MISC_WRITEPROTECT_ON);
	
	return	MISC_SUCCESS;
}

/**
 * @brief	get a PSR status.
 * - 1. get a PSR status and set to "data".
 *
 * @param	*data; unsigned short: 			PSR status
 * -		MISC_PSRINT_LEVEL0 : PSR interrupt level 0
 * -		MISC_PSRINT_LEVEL1 : PSR interrupt level 1
 * -		MISC_PSRINT_LEVEL2 : PSR interrupt level 2
 * -		MISC_PSRINT_LEVEL3 : PSR interrupt level 3
 * -		MISC_PSRINT_LEVEL4 : PSR interrupt level 4
 * -		MISC_PSRINT_LEVEL5 : PSR interrupt level 5
 * -		MISC_PSRINT_LEVEL6 : PSR interrupt level 6
 * -		MISC_PSRINT_LEVEL7 : PSR interrupt level 7
 * -		MISC_PSRIE_DISABLE : PSR interrupt disable
 * -		MISC_PSRIE_ENABLE  : PSR interrupt enable
 * -		MISC_CFLAG_CLEARED : PSR carry(C) flag 0
 * -		MISC_CFLAG_SET     : PSR carry(C) flag 1
 * -		MISC_VFLAG_CLEARED : PSR overflow(V) flag 0
 * -		MISC_VFLAG_SET     : PSR overflow(V) flag 1
 * -		MISC_ZFLAG_CLEARED : PSR zero(Z) flag 0
 * -		MISC_ZFLAG_SET     : PSR zero(Z) flag 1
 * -		MISC_NFLAG_CLEARED : PSR negative(N) flag 0
 * -		MISC_NFLAG_SET     : PSR negative(N) flag 1
 * @return	none
 * @date	2009/12/25
 */
void MISC_getPSR(unsigned short *data)
{
	// PSR status get
	*data = RegRead(R16_MISC_PSR);
}

#endif	// end of PE_MISC
