/**
 * @file	mux.c
 * @brief	MUX driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_MUX
#include <mux_api.h>
#ifdef	PE_ADC
#include <adc_api.h>
#endif	// end of PE_ADC
#ifdef	PE_SPI
#include <spi_api.h>
#endif	// end of PE_SPI
#ifdef	PE_UART
#include <uart_api.h>
#endif	// end of PE_UART
#ifdef	PE_RFC
#include <rfc_api.h>
#endif	// end of PE_RFC
#ifdef	PE_T16E
#include <t16e_api.h>
#endif	// end of PE_T16E
#ifdef	PE_T16
#include <t16_api.h>
#endif	// end of T16
#ifdef	PE_PORT
#include <port_api.h>
#endif	// PE_PORT

/*
 * definition of "define"
 */
#define	MUX_INIT_DATA					0x00								// initialize data


/*
 * definition of constants
 */
struct tag_SET_PORT;
typedef struct tag_SET_PORT {
	volatile unsigned char	*port;											// port data
	unsigned char	mask;													// mask data
	unsigned char	bit;													// bit data
} SET_PORT;

#ifdef	PE_ADC
// ADC port data
static SET_PORT setADC[] = {
	{ MUX_PORT_AIN0,	MUX_MASK_AIN0,	MUX_ADC_AIN0	},
	{ MUX_PORT_AIN1,	MUX_MASK_AIN1,	MUX_ADC_AIN1	},
	{ MUX_PORT_AIN2,	MUX_MASK_AIN2,	MUX_ADC_AIN2	},
	{ MUX_PORT_AIN3,	MUX_MASK_AIN3,	MUX_ADC_AIN3	},
#ifdef	ADC_CHANNEL8
	{ MUX_PORT_AIN4,	MUX_MASK_AIN4,	MUX_ADC_AIN4	},
	{ MUX_PORT_AIN5,	MUX_MASK_AIN5,	MUX_ADC_AIN5	},
	{ MUX_PORT_AIN6,	MUX_MASK_AIN6,	MUX_ADC_AIN6	},
	{ MUX_PORT_AIN7,	MUX_MASK_AIN7,	MUX_ADC_AIN7	}
#endif	// end of ADC_CHANNEL8
};
#endif	// end of PE_ADC

#ifdef	PE_SPI
// SPI port data
static SET_PORT setSPI[][MUX_SPI_PORTMAX] = {
	{
		{ MUX_PORT_SPICLK0,	MUX_MASK_SPICLK0,	MUX_SPI_SPICLK0	},
		{ MUX_PORT_SDO0,	MUX_MASK_SDO0	,	MUX_SPI_SDO0	},
		{ MUX_PORT_SDI0,	MUX_MASK_SDI0	,	MUX_SPI_SDI0	},
		{ MUX_PORT_SPISS0,	MUX_MASK_SPISS0,	MUX_SPI_SPISS0	}
	}
};
#endif	// end of PE_SPI

#ifdef	PE_UART
// UART port data
static SET_PORT setUART[][MUX_UART_PORTMAX] = {
	{																		// channel 0
		{ MUX_PORT_SOUT0,	MUX_MASK_SOUT0,	MUX_UART_SOUT0	},
		{ MUX_PORT_SIN0,	MUX_MASK_SIN0,	MUX_UART_SIN0	},
		{ MUX_PORT_SCLK0,	MUX_MASK_SCLK0,	MUX_UART_SCLK0	}
	},
#ifdef	UART_CHANNEL2
	{																		// channel 1
		{ MUX_PORT_SOUT1,	MUX_MASK_SOUT1,	MUX_UART_SOUT1	},
		{ MUX_PORT_SIN1,	MUX_MASK_SIN1,	MUX_UART_SIN1	},
		{ MUX_PORT_SCLK1,	MUX_MASK_SCLK1,	MUX_UART_SCLK1	}
	}
#endif	// end of UART_CHANNEL2
};
#endif	// end of PE_UART

#ifdef	PE_RFC
// RFC port data
static SET_PORT setRFC[][MUX_RFC_PORTMAX] = {
	{																		// channel 0
		{ MUX_PORT_RFIN0,	MUX_MASK_RFIN0,		MUX_RFC_RFIN0	},
		{ MUX_PORT_REF0,	MUX_MASK_REF0,		MUX_RFC_REF0	},
		{ MUX_PORT_SENA0,	MUX_MASK_SENA0,		MUX_RFC_SENA0	},
		{ MUX_PORT_SENB0,	MUX_MASK_SENB0,		MUX_RFC_SENB0	},
		{ MUX_PORT_RFCLKO,	MUX_MASK_RFCLKO,	MUX_RFC_RFCLKO	}
	},
#ifdef	RFC_CHANNEL2
	{																		// channel 1
		{ MUX_PORT_RFIN1,	MUX_MASK_RFIN1,		MUX_RFC_RFIN1	},
		{ MUX_PORT_REF1,	MUX_MASK_REF1,		MUX_RFC_REF1	},
		{ MUX_PORT_SENA1,	MUX_MASK_SENA1,		MUX_RFC_SENA1	},
		{ MUX_PORT_SENB1,	MUX_MASK_SENB1,		MUX_RFC_SENB1	},
		{ MUX_PORT_RFCLKO,	MUX_MASK_RFCLKO,	MUX_RFC_RFCLKO	}
	}
#endif	// end of RFC_CHANNEL2
};
#endif	// end of PE_RFC

#ifdef	PE_I2CM
// I2C master port data
static SET_PORT setI2CM[][MUX_I2CM_PORTMAX] = {
	{																		// select 0
		{ MUX_PORT_SCL0_0,	MUX_MASK_SCL0_0,	MUX_I2CM_SCL0_0	},
		{ MUX_PORT_SDA0_0,	MUX_MASK_SDA0_0,	MUX_I2CM_SDA0_0	}
	},
	{																		// select 1
		{ MUX_PORT_SCL0_1,	MUX_MASK_SCL0_1,	MUX_I2CM_SCL0_1	},
		{ MUX_PORT_SDA0_1,	MUX_MASK_SDA0_1,	MUX_I2CM_SDA0_1	}
	}
};
#endif	// end of PE_I2CM

#ifdef	PE_T16E
// PWM timer port data
static SET_PORT setT16E[][MUX_T16E_PORTMAX] = {
	{																		// channel 0
		{ MUX_PORT_TOUT3,	MUX_MASK_TOUT3,		MUX_T16E_TOUT3	},
		{ MUX_PORT_TOUTN3,	MUX_MASK_TOUTN3,	MUX_T16E_TOUTN3	}
	},
#ifdef	T16E_CHANNEL2
	{																		// channel 1
		{ MUX_PORT_TOUT4E,	MUX_MASK_TOUT4E,	MUX_T16E_TOUT4E		},
		{ MUX_PORT_TOUTN4E,	MUX_MASK_TOUTN4E,	MUX_T16E_TOUTN4E	}
	}
#endif	// end of T16E_CHANNEL2
};
#endif	// end of PE_T16E

#ifdef	PE_T16
// 16-bit timer port data
static SET_PORT setT16[] = {
	{ MUX_PORT_EXCL0,	MUX_MASK_EXCL0,	MUX_T16_EXCL0	},
	{ MUX_PORT_EXCL1,	MUX_MASK_EXCL1,	MUX_T16_EXCL1	},
	{ MUX_PORT_EXCL2,	MUX_MASK_EXCL2,	MUX_T16_EXCL2	}
};
#endif	// end of PE_T16


/*
 * Declarations of function prototype
 */
#if	defined(PE_ADC) || defined(PE_SPI) || defined(PE_UART) \
	|| defined(PE_I2CM) || defined(PE_T16E) || defined(PE_T16)
static void setPort(SET_PORT *portData, unsigned char sw);
#endif

#ifdef	PE_PORT
void MUX_init(void);
#endif	// end of PE_PORT
#ifdef	PE_REMC
short MUX_setREMCport(unsigned char sw);
#endif	// end of PE_REMC
#ifdef	PE_ADC
short MUX_setADCport(unsigned char use, unsigned short startCh, unsigned short endCh, unsigned char sw);
#endif	// end of PE_ADC
#ifdef	PE_SPI
short MUX_setSPIport(unsigned char chNo, unsigned char sw);
#endif	// end of PE_SPI
#ifdef	PE_UART
short MUX_setUARTport(unsigned char chNo, unsigned char inputClk, unsigned char sw);
#endif	// end of PE_UART
#ifdef	PE_RFC
short MUX_setRFCport(unsigned char chNo, MUX_USE_RFCPORT *use, unsigned char sw);
#endif	// end of PE_RFC
#ifdef	PE_I2CM
short MUX_setI2CMport(unsigned char selectNo, unsigned char sw);
#endif	// end of PE_I2CM
#ifdef	PE_I2CS
short MUX_setI2CSport(unsigned char sw);
#endif	// end of PE_I2CS
#ifdef	PE_OSC
short MUX_setOSCport(unsigned char useFout1, unsigned char useFouth, unsigned char sw);
#endif	// end of PE_OSC
#ifdef	PE_T16E
short MUX_setT16Eport(unsigned char chNo, unsigned char sw);
#endif	// end of PE_T16E
#ifdef	PE_LCD
short MUX_setLCDport(unsigned char sw);
#endif	// end of PE_LCD
#ifdef	PE_T8OSC1
short MUX_setT8OSC1port(unsigned char sw);
#endif	// end of T8OSC1
short MUX_setDBGport(unsigned char sw);
#ifdef	PE_T16
short MUX_setT16port(unsigned char chNo, unsigned char sw);
#endif	// end of T16


#if	defined(PE_ADC) || defined(PE_SPI) || defined(PE_UART) \
	|| defined(PE_I2CM) || defined(PE_T16E) || defined(PE_T16)
/*
 * set a port data
 * param: SET_PORT *portData						port data
 * param: unsigned char sw							select port / function
 * return: void
 * history: 2009/12/25
 */
static void setPort(SET_PORT *portData, unsigned char sw)
{
	// port set
	RegModify(portData->port, portData->mask, (portData->bit * sw));		// use channel port set
}
#endif

#ifdef	PE_PORT
/**
 * @brief	MUX initialize.
 * - 1. set default value in all input/outpu ports.
 *
 * @return	none
 * @date	2009/12/25
 */
void MUX_init(void)
{
	unsigned char	i;														// counter

	// MUX initialize data set
	for(i = 0; i < PORT_NO_MAX; i++) {
		RegWrite((R08_P0L_PMUX + (i * 2)), MUX_INIT_DATA);					// Px[3:0] initialize
		RegWrite((R08_P0H_PMUX + (i * 2)), MUX_INIT_DATA);					// Px[7:4] initialize
	}
}
#endif	// end of PE_PORT

#ifdef	PE_REMC
/**
 * @brief	set REMC ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set REMC ports in the case of MUX_ON.
 *
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set REMC ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setREMCport(unsigned char sw)
{
	// parameter check
	if(sw > MUX_ON) {
		return	MUX_INVALID_PARAMETER;
	}

	// REMC port set
	RegModify(MUX_PORT_REMO, MUX_MASK_REMO, (MUX_REMC_REMO * sw));			// REMO port set
	RegModify(MUX_PORT_REMI, MUX_MASK_REMI, (MUX_REMC_REMI * sw));			// REMI port set

	return	MUX_SUCCESS;
}
#endif	// end of PE_REMC

#ifdef	PE_ADC
/**
 * @brief	set ADC ports.
 * - 1. check parameters.
 * - 2. set a xADTRG in the case of use external trigger.
 * - 3. set input/output ports in the case of MUX_OFF,
 *		set ADC ports in the case of MUX_ON.
 *		(from conversion start channel to conversion end channel.)
 *
 * @param	use; unsigned char: 			select external trigger
 * -		MUX_ADTRG_OFF : external trigger is not use
 * -		MUX_ADTRG_ON  : external trigger is use
 * @param	startCh; unsigned short: 		select start channel number
 * -		ADC_CH0 : conversion start channel number 0
 * -		ADC_CH1 : conversion start channel number 1
 * -		ADC_CH2 : conversion start channel number 2
 * -		ADC_CH3 : conversion start channel number 3
 * -		ADC_CH4 : conversion start channel number 4
 * -		ADC_CH5 : conversion start channel number 5
 * -		ADC_CH6 : conversion start channel number 6
 * -		ADC_CH7 : conversion start channel number 7
 * @param	endCh; unsigned short: 			select end channel number
 * -		ADC_CH0 : conversion end channel number 0
 * -		ADC_CH1 : conversion end channel number 1
 * -		ADC_CH2 : conversion end channel number 2
 * -		ADC_CH3 : conversion end channel number 3
 * -		ADC_CH4 : conversion end channel number 4
 * -		ADC_CH5 : conversion end channel number 5
 * -		ADC_CH6 : conversion end channel number 6
 * -		ADC_CH7 : conversion end channel number 7
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set ADC ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setADCport(unsigned char use, unsigned short startCh, unsigned short endCh, unsigned char sw)
{
	unsigned short	ch;														// select channel counter
	SET_PORT		*portData = &setADC[startCh];							// port data

	// parameter check
	if(	(use > MUX_ADTRG_ON) ||
		(startCh >= ADC_CH_MAX) ||
		(endCh >= ADC_CH_MAX) ||
		(startCh > endCh) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// ADC port set
	if(use == MUX_ADTRG_ON) {												// select external trigger
		RegModify(MUX_PORT_ADTRG, MUX_MASK_ADTRG, (MUX_ADC_ADTRG * sw));
	}

	for(ch = startCh; ch <= endCh; ch++) {
		setPort(portData, sw);											// use channel port set
		++portData;
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_ADC

#ifdef	PE_SPI
/**
 * @brief	set SPI ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set SPI ports in the case of MUX_ON.
 *
 * @param	chNo; unsigned char: 			select channel number
 * -		SPI_CH0 : SPI channel number 0
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set SPI ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setSPIport(unsigned char chNo, unsigned char sw)
{
	unsigned char	port;													// port data number
	SET_PORT		*portData = &setSPI[chNo][0];							// port data

	// parameter check
	if(	(chNo >= SPI_CH_MAX) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// SPI port set
	for(port = 0; port < MUX_SPI_PORTMAX; port++) {
		setPort(portData, sw);
		++portData;
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_SPI

#ifdef	PE_UART
/**
 * @brief	set UART ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set UART ports in the case of MUX_ON.
 *		(set a SCLK in the case of use external clock.)
 *
 * @param	chNo; unsigned char: 			select channel number
 * -		UART_CH0 : UART channel number 0
 * -		UART_CH1 : UART channel number 1
 * @param	inputClk; unsigned char: 		select input clock
 * -		MUX_UART_CLK_INTERNAL internal clock
 * -		MUX_UART_CLK_EXTERNAL external clock
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set UART ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setUARTport(unsigned char chNo, unsigned char inputClk, unsigned char sw)
{
	unsigned char	port;													// port data number
	SET_PORT		*portData = &setUART[chNo][0];							// port data

	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		(inputClk > MUX_UART_CLK_EXTERNAL) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// UART port set
	for(port = 0; port < (MUX_UART_PORTMAX - 1); port++) {					// set SCLK only external clock use
		setPort(portData, sw);
		++portData;
	}
	if(inputClk == MUX_UART_CLK_EXTERNAL) {
		setPort(portData, sw);												// use external clock
		++portData;
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_UART

#ifdef	PE_RFC
/**
 * @brief	set RFC ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set RFC ports in the case of MUX_ON.
 *
 * @param	chNo; unsigned char: 			select channel number
 * -		RFC_CH0 : RFC channel number 0
 * -		RFC_CH1 : RFC channel number 1
 * @param	*use; MUX_USE_RFCPORT: 			select use port
 * -		useRfin;  unsigned char:		select external clock
 * 	-			MUX_RFIN_OFF : external clock is not use
 * 	-			MUX_RFIN_ON  : external clock is use
 * -		useRef; unsigned char:			select reference
 * 	-			MUX_REF_OFF : reference is not use
 * 	-			MUX_REF_ON  : reference is use
 * -		useSenA; unsigned char:			select sensor A
 * 	-			MUX_SENA_OFF : sensor A is not use
 * 	-			MUX_SENA_ON  : sensor A is use
 * -		useSenB; unsigned char:			select sensor B
 * 	-			MUX_SENB_OFF : sensor B is not use
 * 	-			MUX_SENB_ON  : sensor B is use
 * -		useRfclk; unsigned char:		select RFCK monitor output
 * 	-			MUX_RFCLK_OFF : RFCK monitor output is not use
 * 	-			MUX_RFCLK_ON  : RFCK monitor output is use
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set RFC ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setRFCport(unsigned char chNo, MUX_USE_RFCPORT *use, unsigned char sw)
{
	SET_PORT	*portData = &setRFC[chNo][0];								// port data

	// parameter check
	if(	(chNo >= RFC_CH_MAX) ||
		(use->useRfin > MUX_RFIN_ON) ||
		(use->useRef > MUX_REF_ON) ||
		(use->useSenA > MUX_SENA_ON) ||
		(use->useSenB > MUX_SENB_ON) ||
		(use->useRfclk > MUX_RFCLK_ON) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// RFC port set
	if(use->useRfin == MUX_RFIN_ON) {
		RegModify(portData->port, portData->mask, (portData->bit * sw));
	}
	portData++;
	if(use->useRef == MUX_REF_ON) {
		RegModify(portData->port, portData->mask, (portData->bit * sw));
	}
	portData++;
	if(use->useSenA == MUX_SENA_ON) {
		RegModify(portData->port, portData->mask, (portData->bit * sw));
	}
	portData++;
	if(use->useSenB == MUX_SENB_ON) {
		RegModify(portData->port, portData->mask, (portData->bit * sw));
	}
	portData++;
	if(use->useRfclk == MUX_RFCLK_ON) {
		RegModify(portData->port, portData->mask, (portData->bit * sw));
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_RFC

#ifdef	PE_I2CM
/**
 * @brief	set I2C master ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set I2C master ports in the case of MUX_ON.
 *
 * @param	selectNo; unsigned char: 		select number
 * -		MUX_I2CM_0 : port select number 0
 * -		MUX_I2CM_1 : port select number 1
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set I2C master ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setI2CMport(unsigned char selectNo, unsigned char sw)
{
	unsigned char	port;													// port data number
	SET_PORT		*portData = &setI2CM[selectNo][0];						// port data

	// parameter check
	if(	(selectNo > MUX_I2CM_1) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// I2C master port set
	for(port = 0; port < MUX_I2CM_PORTMAX; port++) {
		setPort(portData, sw);
		++portData;
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_I2CM

#ifdef	PE_I2CS
/**
 * @brief	set I2C slave ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set I2C slave ports in the case of MUX_ON.
 *
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set I2C slave ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setI2CSport(unsigned char sw)
{
	// parameter check
	if(sw > MUX_ON) {
		return	MUX_INVALID_PARAMETER;
	}

	// I2C slave port set
	RegModify(MUX_PORT_SCL1, MUX_MASK_SCL1, (MUX_I2CS_SCL1 * sw));
	RegModify(MUX_PORT_SDA1, MUX_MASK_SDA1, (MUX_I2CS_SDA1 * sw));
	RegModify(MUX_PORT_BFR,  MUX_MASK_BFR,  (MUX_I2CS_BFR  * sw));

	return	MUX_SUCCESS;
}
#endif	// end of PE_I2CS

#ifdef	PE_OSC
/**
 * @brief	set OSC ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set OSC ports in the case of MUX_ON.
 * - 3. set OSC3/OSC4 ports in the case of S1C17611.
 *
 * @param	useFout1; unsigned char: 		select FOUT1
 * -		MUX_FOUT1_OFF : FOUT1 is not use
 * -		MUX_FOUT1_ON  : FOUT1 is use
 * @param	useFouth; unsigned char: 		select FOUTH
 * -		MUX_FOUTH_OFF : FOUTH is not use
 * -		MUX_FOUTH_ON  : FOUTH is use
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set OSC ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setOSCport(unsigned char useFout1, unsigned char useFouth, unsigned char sw)
{
	// parameter check
	if(	(useFout1 > MUX_FOUT1_ON) ||
		(useFouth > MUX_FOUTH_ON) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

#ifdef	S1C17611
	// OSC3/OSC4 port set
	RegModify(MUX_PORT_OSC3, MUX_MASK_OSC3, MUX_OSC_OSC3);
	RegModify(MUX_PORT_OSC4, MUX_MASK_OSC4, MUX_OSC_OSC4);
#endif	// end of S1C17611

	// OSC port set
	if(useFout1 == MUX_FOUT1_ON) {
		RegModify(MUX_PORT_FOUT1, MUX_MASK_FOUT1, (MUX_OSC_FOUT1 * sw));
	}
	if(useFouth == MUX_FOUTH_ON) {
		RegModify(MUX_PORT_FOUTH, MUX_MASK_FOUTH, (MUX_OSC_FOUTH * sw));
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_OSC

#ifdef	PE_T16E
/**
 * @brief	set PWM timer ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set PWM timer ports in the case of MUX_ON.
 *
 * @param	chNo; unsigned char: 			select channel number
 * -		T16E_CH0 : PWM timer channel number 0
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set PWM timer ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setT16Eport(unsigned char chNo, unsigned char sw)
{
	unsigned char	port;													// port data number
	SET_PORT		*portData = &setT16E[chNo][0];							// port data

	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// PWM timer port set
	for(port = 0; port < MUX_T16E_PORTMAX; port++) {
		setPort(portData, sw);
		++portData;
	}

	return	MUX_SUCCESS;
}
#endif	// end of PE_T16E

#ifdef	PE_LCD
/**
 * @brief	set LCD ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set LCD ports in the case of MUX_ON.
 *
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set LCD ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setLCDport(unsigned char sw)
{
	// parameter check
	if(sw > MUX_ON) {
		return	MUX_INVALID_PARAMETER;
	}

	// LCD port set
	RegModify(MUX_PORT_LFRO, MUX_MASK_LFRO, (MUX_LCD_LFRO * sw));

	return	MUX_SUCCESS;
}
#endif	// end of PE_LCD

#ifdef	PE_T8OSC1
/**
 * @brief	set 8-bit OSC1 timer ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set 8-bit OSC1 timer ports in the case of MUX_ON.
 *
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set 8-bit OSC1 timer ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setT8OSC1port(unsigned char sw)
{
	// parameter check
	if(sw > MUX_ON) {
		return	MUX_INVALID_PARAMETER;
	}

	// 8-bit OSC1 port set
	RegModify(MUX_PORT_TOUT4, MUX_MASK_TOUT4, (MUX_T8OSC1_TOUT4 * sw));

	return	MUX_SUCCESS;
}
#endif	// end of PE_T8OSC1

/**
 * @brief	set debug ports.
 * - 1. check parameters.
 * - 2. set debug ports in the case of MUX_OFF,
 *		set input/output ports in the case of MUX_ON.
 *
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set debug ports
 * -		MUX_ON  : set input/output ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setDBGport(unsigned char sw)
{
	// parameter check
	if(sw > MUX_ON) {
		return	MUX_INVALID_PARAMETER;
	}

	// debug port set
	RegModify(MUX_PORT_P41, MUX_MASK_P41, (MUX_DBG_P41 * sw));
	RegModify(MUX_PORT_P42, MUX_MASK_P42, (MUX_DBG_P42 * sw));
#ifndef	S1C17611
	RegModify(MUX_PORT_P43, MUX_MASK_P43, (MUX_DBG_P43 * sw));
#endif	// end of S1C17611

	return	MUX_SUCCESS;
}

#ifdef	PE_T16
/**
 * @brief	set 16-bit timer ports.
 * - 1. check parameters.
 * - 2. set input/output ports in the case of MUX_OFF,
 *		set 16-bit timer ports in the case of MUX_ON.
 *
 * @param	chNo; unsigned char: 			select channel number
 * -		T16_CH0 : 16-bit timer channel number 0
 * -		T16_CH1 : 16-bit timer channel number 1
 * -		T16_CH2 : 16-bit timer channel number 2
 * @param	sw; unsigned char: 				select port function
 * -		MUX_OFF : set input/output ports
 * -		MUX_ON  : set 16-bit timer ports
 * @return	short:							end status
 * -		MUX_SUCCESS : success
 * -		MUX_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short MUX_setT16port(unsigned char chNo, unsigned char sw)
{
	SET_PORT	*portData = &setT16[chNo];									// port data

	// parameter check
	if(	(chNo >= T16_CH_MAX) ||
		(sw > MUX_ON) ) {
			return	MUX_INVALID_PARAMETER;
	}

	// 16-bit timer port set
	setPort(portData, sw);

	return	MUX_SUCCESS;
}
#endif	// end of PE_T16

#endif	// end of PE_MUX
