/**
 * @file	osc.c
 * @brief	osc driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#include <osc_api.h>
#include <osc.h>

#ifdef	PE_OSC

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static const unsigned char	selectSysClk[] = {
	IOSCEN,																	// select IOSC
	OSC1EN,																	// select OSC1
	OSC3EN,																	// select OSC3
	OSC1EN																	// select OSC1
};

/*
 * Declarations of function prototype
 */
short OSC_setClockSource(unsigned char selectHSCLK, unsigned char selectCLKSRC);
short OSC_setWaitCycle(unsigned char selectIOSC, unsigned char selectOSC3);
short OSC_controlOscillation(unsigned char selectBit);
short OSC_setNoiseFilter(unsigned char selectBit);
short OSC_setLCDClock(unsigned char selectOsc, unsigned char selectRatio);
short OSC_controlLCDClock(unsigned char selectBit);
short OSC_setFOUTHDivision(unsigned char selectBit);
short OSC_controlFOUT(unsigned char selectBit);
short OSC_setT8OSC1Division(unsigned char selectBit);
short OSC_controlT8OSC1(unsigned char selectBit);
short OSC_setSVDClock(unsigned char selectBit);
short OSC_controlSVD(unsigned char selectBit);
short OSC_setRFCClock(unsigned char selectOsc, unsigned char selectRatio);
short OSC_controlRFC(unsigned char selectBit);


/**
 * @brief	set a clock source.
 * - 1. check parameters.
 * - 2. set a selected clock.
 *
 * @param	selectHSCLK; unsigned char: 	select High-speed clock
 * -		OSC_SELHSCLK_IOSC : IOSC
 * -		OSC_SELHSCLK_OSC3 : OSC3
 * @param	selectCLKSRC; unsigned char: 	select system clock source
 * -		OSC_SELCLKSRC_HSCLK : HSCLK
 * -		OSC_SELCLKSRC_OSC1  : OSC1
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setClockSource(unsigned char selectHSCLK, unsigned char selectCLKSRC)
{
	// parameter check
	if(	(selectHSCLK > OSC_SELHSCLK_OSC3) ||
		(selectCLKSRC > OSC_SELCLKSRC_OSC1) ) {
			return	OSC_INVALID_PARAMETER;
	}
	
	// set clock source
	RegWrite(R08_OSC_SRC, ((selectHSCLK << HSCLKSEL_SFT) | (selectCLKSRC << CLKSRC_SFT)));
	
	return	OSC_SUCCESS;
}

/**
 * @brief	set an oscillation stable wait cycle.
 * - 1. check parameters.
 * - 2. set a selected stable wait cycle.
 *
 * @param	selectIOSC; unsigned char: 		select IOSC wait cycle
 * -		OSC_SELIOSC_64 : 64 cycle
 * -		OSC_SELIOSC_32 : 32 cycle
 * -		OSC_SELIOSC_16 : 16 cycle
 * -		OSC_SELIOSC_8  :  8 cycle
 * @param	selectOSC3; unsigned char: 		select OSC3 wait cycle
 * -		OSC_SELOSC3_1024 : 1024 cycle
 * -		OSC_SELOSC3_512  :  512 cycle
 * -		OSC_SELOSC3_256  :  256 cycle
 * -		OSC_SELOSC3_128  :  128 cycle
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setWaitCycle(unsigned char selectIOSC, unsigned char selectOSC3)
{
	// parameter check
	if(	(selectIOSC >= OSC_SELIOSC_MAX) ||
		(selectOSC3 >= OSC_SELOSC3_MAX) ) {
			return	OSC_INVALID_PARAMETER;
	}
	
	// set oscillation stable waiting cycle
	RegModify(R08_OSC_CTL, (IOSCWT | OSC3WT), ((selectIOSC << IOSCWT_SFT) | (selectOSC3 << OSC3WT_SFT)));
	
	return	OSC_SUCCESS;
}

/**
 * @brief	oscillation enable / disable.
 * - 1. check parameters.
 * - 2. set an oscillation enable/disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		OSC_IOSC_DISABLE : IOSC oscillation disable
 * -		OSC_IOSC_ENABLE  : IOSC oscillation enable
 * -		OSC_OSC1_DISABLE : OSC1 oscillation disable
 * -		OSC_OSC1_ENABLE  : OSC1 oscillation enable
 * -		OSC_OSC3_DISABLE : OSC3 oscillation disable
 * -		OSC_OSC3_ENABLE  : OSC3 oscillation enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_FAILURE : oscillation stop failure 
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlOscillation(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSCEN_MSK) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// system clock check
	if((selectBit & selectSysClk[(RegRead(R08_OSC_SRC) & (HSCLKSEL | CLKSRC))]) == OSC_DISABLE ) {
		return	OSC_FAILURE;
	}
	
	// oscillation enable / disable
	RegModify(R08_OSC_CTL, OSCEN_MSK, selectBit);
	
	return	OSC_SUCCESS;
}

/**
 * @brief	noise filter enable / disable.
 * - 1. check parameters.
 * - 2. set a noise filter enable/disable.
 *
 * @param	selectBit; unsigned char: 		select noise filter enable / disable
 * -		OSC_NOISE_RESET_DISABLE : noise filter disable for RESET input
 * -		OSC_NOISE_RESET_ENABLE  : noise filter enable for RESET input
 * -		OSC_NOISE_NMI_DISABLE   : noise filter disable for NMI input
 * -		OSC_NOISE_NMI_ENABLE    : noise filter enable for NMI input
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setNoiseFilter(unsigned char selectBit)
{
	// parameter check
	if(selectBit > NFEN_MSK) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// noise filter enable / disable
	RegWrite(R08_OSC_NFEN, selectBit);
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_OSC


#ifdef	PE_LCD
/**
 * @brief	set a LCD clock.
 * - 1. check parameters.
 * - 2. disable LCD clock.
 * - 3. set a LCD clock source.
 * - 4. set a division ratio, if a LCD clock source is HSCLK.
 * - 5. enable LCD clock.
 *
 * @param	selectOsc; unsigned char: 		select LCD clock source
 * -		OSC_LCDCLK_HSCLK : HSCLK
 * -		OSC_LCDCLK_OSC1  : OSC1
 * @param	selectRatio; unsigned char: 	select LCD clock division ratio, if a LCD clock source is HSCLK
 * -		OSC_LCDRATIO_32  : HSCLK 1/32
 * -		OSC_LCDRATIO_64  : HSCLK 1/64
 * -		OSC_LCDRATIO_128 : HSCLK 1/128
 * -		OSC_LCDRATIO_256 : HSCLK 1/256
 * -		OSC_LCDRATIO_512 : HSCLK 1/512
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setLCDClock(unsigned char selectOsc, unsigned char selectRatio)
{
	// parameter check
	if(	(selectOsc > OSC_LCDCLK_OSC1) ||
		(selectRatio >= OSC_LCDRATIO_MAX) ) {
			return	OSC_INVALID_PARAMETER;
	}
	
	// LCD clock disable
	OSC_controlLCDClock(OSC_LCDCLK_DISABLE);
	
	// LCD clock source set
	RegModify(R08_OSC_LCLK, LCKSRC, (selectOsc << LCKSRC_SFT));
	
	// LCD clock division ratio set for HSCLK
	if(selectOsc == OSC_LCDCLK_HSCLK) {
		RegModify(R08_OSC_LCLK, LCKDV, (selectRatio << LCKDV_SFT));
	}
	
	// LCD clock enable
	OSC_controlLCDClock(OSC_LCDCLK_ENABLE);
	
	return	OSC_SUCCESS;
}

/**
 * @brief	LCD clock enable / disable.
 * - 1. check parameters.
 * - 2. set a LCD clock enable/disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		OSC_LCDCLK_DISABLE : LCD clock disable
 * -		OSC_LCDCLK_ENABLE  : LCD clock enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlLCDClock(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSC_LCDCLK_ENABLE) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// LCD clock enable / disable
	RegModify(R08_OSC_LCLK, LCKEN, (selectBit << LCDEN_SFT));
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_LCD


#ifdef	PE_OSC
/**
 * @brief	set a FOUTH clock division ratio.
 * - 1. check parameters.
 * - 2. set a FOUTH clock division ratio.
 *
 * @param	selectBit; unsigned char: 		select FOUTH clock division ratio
 * -		OSC_FOUTCLK_1 : HSCLK 1/1
 * -		OSC_FOUTCLK_2 : HSCLK 1/2
 * -		OSC_FOUTCLK_4 : HSCLK 1/4
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setFOUTHDivision(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= OSC_FOUTCLK_MAX) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// FOUT clock output disable
	OSC_controlFOUT((RegRead(R08_OSC_FOUT) & FOUT1E ) | OSC_FOUTH_DISABLE);
	
	// set FOUTH clock division ratio
	RegModify(R08_OSC_FOUT, FOUTHD, (selectBit << FOUTHD_SFT));
	
	return	OSC_SUCCESS;
}

/**
 * @brief	FOUT clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a FOUT clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select FOUT clock output enable / disable
 * -		OSC_FOUTH_DISABLE : FOUTH clock output disable
 * -		OSC_FOUTH_ENABLE  : FOUTH clock output enable
 * -		OSC_FOUT1_DISABLE : FOUT1 clock output disable
 * -		OSC_FOUT1_ENABLE  : FOUT1 clock output enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlFOUT(unsigned char selectBit)
{
	// parameter check
	if(selectBit > FOUT_MSK) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// FOUT clock enable / disable
	RegModify(R08_OSC_FOUT, FOUT_MSK, selectBit);
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_OSC


#ifdef	PE_T8OSC1
/**
 * @brief	set a T8OSC1 clock division ratio.
 * - 1. check parameters.
 * - 2. set a T8OSC1 clock division ratio.
 *
 * @param	selectBit; unsigned char: 		select T8OSC1 clock division ratio
 * -		OSC_T8OSC1CLK_1  : HSCLK 1/1
 * -		OSC_T8OSC1CLK_2  : HSCLK 1/2
 * -		OSC_T8OSC1CLK_4  : HSCLK 1/4
 * -		OSC_T8OSC1CLK_8  : HSCLK 1/8
 * -		OSC_T8OSC1CLK_16 : HSCLK 1/16
 * -		OSC_T8OSC1CLK_32 : HSCLK 1/32
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setT8OSC1Division(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= OSC_T8OSC1CLK_MAX) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// T8OSC1 clock output disable
	OSC_controlT8OSC1(OSC_T8OSC1_DISABLE);
	
	// set T8OSC1 clock division ratio
	RegModify(R08_OSC_T8OSC1, T8O1CK, (selectBit << T8O1CK_SFT));
	
	return	OSC_SUCCESS;
}

/**
 * @brief	T8OSC1 clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a T8OSC1 clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select T8OSC1 clock output enable / disable
 * -		OSC_T8OSC1_DISABLE : T8OSC1 clock output disable
 * -		OSC_T8OSC1_ENABLE  : T8OSC1 clock output enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlT8OSC1(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSC_T8OSC1_ENABLE) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// T8OSC1 clock enable / disable
	RegModify(R08_OSC_T8OSC1, T8O1CE, (selectBit << T8O1CE_SFT));
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_T8OSC1


#ifdef	PE_SVD
/**
 * @brief	set a SVD clock.
 * - 1. check parameters.
 * - 2. set a SVD clock source.
 *
 * @param	selectBit; unsigned char: 		select SVD clock source
 * -		OSC_SVDCLK_HSCLK : HSCLK 1/512
 * -		OSC_SVDCLK_OSC1  : OSC1
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setSVDClock(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSC_SVDCLK_OSC1) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// SVD clock disable
	OSC_controlSVD(OSC_SVD_DISABLE);
	
	// set SVD clock source
	RegModify(R08_OSC_SVD, SVDSRC, (selectBit << SVDSRC_SFT));
	
	return	OSC_SUCCESS;
}

/**
 * @brief	SVD clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a SVD clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select SVD clock output enable / disable
 * -		OSC_SVD_DISABLE : SVD clock output disable
 * -		OSC_SVD_ENABLE  : SVD clock output enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlSVD(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSC_SVD_ENABLE) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// SVD clock enable / disable
	RegModify(R08_OSC_SVD, SVDCKEN, (selectBit << SVDCKEN_SFT));
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_SVD


#ifdef	PE_RFC
/**
 * @brief	set a RFC clock.
 * - 1. check parameters.
 * - 2. disable RFC clock.
 * - 3. set a RFC clock source.
 * - 4. set a division ratio, if a RFC clock source is HSCLK.
 * - 5. enable RFC clock.
 *
 * @param	selectOsc; unsigned char: 		select RFC clock source
 * -		OSC_RFCCLK_HSCLK : HSCLK
 * -		OSC_RFCCLK_OSC1  : OSC1
 * @param	selectRatio; unsigned char: 	select RFC clock division ratio, if a RFC clock source is HSCLK
 * -		OSC_RFCRATIO_1 : HSCLK 1/1
 * -		OSC_RFCRATIO_2 : HSCLK 1/2
 * -		OSC_RFCRATIO_4 : HSCLK 1/4
 * -		OSC_RFCRATIO_8 : HSCLK 1/8
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_setRFCClock(unsigned char selectOsc, unsigned char selectRatio)
{
	// parameter check
	if(	(selectOsc > OSC_RFCCLK_OSC1) ||
		(selectRatio >= OSC_RFCRATIO_MAX) ) {
			return	OSC_INVALID_PARAMETER;
	}
	
	// RFC clock disable
	OSC_controlRFC(OSC_RFCCLK_DISABLE);
	
	// RFC clock source set
	RegModify(R08_OSC_RFC, RFTCKSRC, (selectOsc << RFTCKSRC_SFT));
	
	// RFC clock division ratio set for HSCLK
	if(selectOsc == OSC_RFCCLK_HSCLK) {
		RegModify(R08_OSC_RFC, RFTCKDV, (selectRatio << RFTCKDV_SFT));
	}
	
	// RFC clock enable
	OSC_controlRFC(OSC_RFCCLK_ENABLE);
	
	return	OSC_SUCCESS;
}

/**
 * @brief	RFC clock enable / disable.
 * - 1. check parameters.
 * - 2. set a RFC clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select RFC clock output enable / disable
 * -		OSC_RFCCLK_DISABLE : RFC clock output disable
 * -		OSC_RFCCLK_ENABLE  : RFC clock output enable
 * @return	short:							end status
 * -		OSC_SUCCESS : success
 * -		OSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short OSC_controlRFC(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSC_RFCCLK_ENABLE) {
		return	OSC_INVALID_PARAMETER;
	}
	
	// RFC clock enable / disable
	RegModify(R08_OSC_RFC, RFTCKEN, (selectBit << RFTCKEN_SFT));
	
	return	OSC_SUCCESS;
}

#endif	// end of PE_RFC
