/**
 * @file	port.c
 * @brief	port driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_PORT
#include <port_api.h>

/*
 * definition of "define"
 */
#define	PORT_INIT_DATA					(0x0000)							// all bit 0
#define PORT_CHAT_LH					(0x01)								// select data position

/*
 * definition of constants
 */
static const unsigned char	PORT_RSVMSK_DATA[] = {
	PORT_RSVMSK_P0,															// P0 port
	PORT_RSVMSK_P1,															// P1 port
	PORT_RSVMSK_P2,															// P2 port
	PORT_RSVMSK_P3,															// P3 port
	PORT_RSVMSK_P4															// P4 port
};

static const unsigned char	PORT_SMTMSK_DATA[] = {
	PORT_SMTMSK_P0,															// P0 port
	PORT_SMTMSK_P1,															// P1 port
	PORT_SMTMSK_P2,															// P2 port
	PORT_SMTMSK_P3,															// P3 port
	PORT_SMTMSK_P4															// P4 port
};

static volatile unsigned char	*PORT_INTLVL_REG[] = {
	PORT_INTLVL_P0,															// P0 port
	PORT_INTLVL_P1															// P1 port
};

/*
 * Declarations of function prototype
 */
void PORT_init(void);
short PORT_getInputData(unsigned char portNo, unsigned char *data);
short PORT_setOutputData(unsigned char portNo, unsigned char data);
short PORT_controlInput(unsigned char portNo, unsigned char selectBit);
short PORT_controlOutput(unsigned char portNo, unsigned char selectBit);
short PORT_controlPullup(unsigned char portNo, unsigned char selectBit);
short PORT_controlSchmittTrigger(unsigned char portNo, unsigned char selectBit);
short PORT_initInt(unsigned char portNo, unsigned char intLevel);
short PORT_controlInt(unsigned char portNo, unsigned char selectBit);
short PORT_setIntEdge(unsigned char portNo, unsigned char selectBit);
short PORT_resetIntFlag(unsigned char portNo, unsigned char selectBit);
short PORT_checkIntFlag(unsigned char portNo, unsigned char *flag);
short PORT_setChatteringFilter(unsigned char portNo, unsigned char selectBit);


/**
 * @brief	Px ports initialize.
 * - 1. all ports initialize.
 *
 * @return	none
 * @date	2009/12/25
 */
void PORT_init(void)
{
	unsigned char	i;														// counter
	
	// interrupt all disable
	for(i = 0; i <= PORT_INT_MAX; i++) {
		PORT_controlInt(PORT_NO_P0 + i, (PORT_PX7_DISABLE | PORT_PX6_DISABLE | PORT_PX5_DISABLE |
			PORT_PX4_DISABLE | PORT_PX3_DISABLE | PORT_PX2_DISABLE | PORT_PX1_DISABLE | PORT_PX0_DISABLE));
	}
	
	// port initialize
	for(i = 0; i < PORT_NO_MAX; i++) {
		// output all disable
		PORT_controlOutput(PORT_NO_P0 + i, (PORT_PX7_DISABLE | PORT_PX6_DISABLE | PORT_PX5_DISABLE |
			PORT_PX4_DISABLE | PORT_PX3_DISABLE | PORT_PX2_DISABLE | PORT_PX1_DISABLE | PORT_PX0_DISABLE));
		
		// input all enable
		PORT_controlInput(PORT_NO_P0 + i, (PORT_PX7_ENABLE | PORT_PX6_ENABLE | PORT_PX5_ENABLE |
			PORT_PX4_ENABLE | PORT_PX3_ENABLE | PORT_PX2_ENABLE | PORT_PX1_ENABLE | PORT_PX0_ENABLE));
		
		// pull-up all enable
		PORT_controlPullup(PORT_NO_P0 + i, (PORT_PX7_ENABLE | PORT_PX6_ENABLE | PORT_PX5_ENABLE |
			PORT_PX4_ENABLE | PORT_PX3_ENABLE | PORT_PX2_ENABLE | PORT_PX1_ENABLE | PORT_PX0_ENABLE));
		
		// output data all 0
		PORT_setOutputData(PORT_NO_P0 + i, PORT_INIT_DATA);
	}
}

/**
 * @brief	Px ports data input.
 * - 1. check parameters.
 * - 2. get an input data from register and set the data to "data".
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * -		PORT_NO_P3 : port 3
 * -		PORT_NO_P4 : port 4
 * @param	*data; unsigned char: 			input port data
 * -		PORT_BIT_X0 : bit 0
 * -		PORT_BIT_X1 : bit 1
 * -		PORT_BIT_X2 : bit 2
 * -		PORT_BIT_X3 : bit 3
 * -		PORT_BIT_X4 : bit 4
 * -		PORT_BIT_X5 : bit 5
 * -		PORT_BIT_X6 : bit 6
 * -		PORT_BIT_X7 : bit 7
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_getInputData(unsigned char portNo, unsigned char *data)
{
	// parameter check
	if(portNo >= PORT_NO_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// get input data
	*data = RegRead(R08_PX_IN(portNo));
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports data output.
 * - 1. check parameters.
 * - 2. set an output data.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * -		PORT_NO_P3 : port 3
 * -		PORT_NO_P4 : port 4
 * @param	data; unsigned char: 			output port data
 * -		PORT_BIT_X0 : bit 0
 * -		PORT_BIT_X1 : bit 1
 * -		PORT_BIT_X2 : bit 2
 * -		PORT_BIT_X3 : bit 3
 * -		PORT_BIT_X4 : bit 4
 * -		PORT_BIT_X5 : bit 5
 * -		PORT_BIT_X6 : bit 6
 * -		PORT_BIT_X7 : bit 7
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_setOutputData(unsigned char portNo, unsigned char data)
{
	// parameter check
	if(portNo >= PORT_NO_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reserve bit mask
	data &= PORT_RSVMSK_DATA[portNo];
	
	// set output data
	RegWrite(R08_PX_OUT(portNo), data);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports input enable / disable.
 * - 1. check parameters.
 * - 2. set an input enable/disable.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * -		PORT_NO_P3 : port 3
 * -		PORT_NO_P4 : port 4
 * @param	selectBit; unsigned char:		select input enable / disable
 * -		PORT_PX0_DISABLE : bit 0 disable
 * -		PORT_PX0_ENABLE  : bit 0 enable
 * -		PORT_PX1_DISABLE : bit 1 disable
 * -		PORT_PX1_ENABLE  : bit 1 enable
 * -		PORT_PX2_DISABLE : bit 2 disable
 * -		PORT_PX2_ENABLE  : bit 2 enable
 * -		PORT_PX3_DISABLE : bit 3 disable
 * -		PORT_PX3_ENABLE  : bit 3 enable
 * -		PORT_PX4_DISABLE : bit 4 disable
 * -		PORT_PX4_ENABLE  : bit 4 enable
 * -		PORT_PX5_DISABLE : bit 5 disable
 * -		PORT_PX5_ENABLE  : bit 5 enable
 * -		PORT_PX6_DISABLE : bit 6 disable
 * -		PORT_PX6_ENABLE  : bit 6 enable
 * -		PORT_PX7_DISABLE : bit 7 disable
 * -		PORT_PX7_ENABLE  : bit 7 enable
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_controlInput(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo >= PORT_NO_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reserve bit mask
	selectBit &= PORT_RSVMSK_DATA[portNo];
	
	// set input enable
	RegWrite(R08_PX_IEN(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports output enable / disable.
 * - 1. check parameters.
 * - 2. set an output enable/disable.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * -		PORT_NO_P3 : port 3
 * -		PORT_NO_P4 : port 4
 * @param	selectBit; unsigned char:		select output enable / disable
 * -		PORT_PX0_DISABLE : bit 0 disable
 * -		PORT_PX0_ENABLE  : bit 0 enable
 * -		PORT_PX1_DISABLE : bit 1 disable
 * -		PORT_PX1_ENABLE  : bit 1 enable
 * -		PORT_PX2_DISABLE : bit 2 disable
 * -		PORT_PX2_ENABLE  : bit 2 enable
 * -		PORT_PX3_DISABLE : bit 3 disable
 * -		PORT_PX3_ENABLE  : bit 3 enable
 * -		PORT_PX4_DISABLE : bit 4 disable
 * -		PORT_PX4_ENABLE  : bit 4 enable
 * -		PORT_PX5_DISABLE : bit 5 disable
 * -		PORT_PX5_ENABLE  : bit 5 enable
 * -		PORT_PX6_DISABLE : bit 6 disable
 * -		PORT_PX6_ENABLE  : bit 6 enable
 * -		PORT_PX7_DISABLE : bit 7 disable
 * -		PORT_PX7_ENABLE  : bit 7 enable
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_controlOutput(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo >= PORT_NO_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reserve bit mask
	selectBit &= PORT_RSVMSK_DATA[portNo];
	
	// set output enable
	RegWrite(R08_PX_OEN(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports pull-up enable / disable.
 * - 1. check parameters.
 * - 2. set a pull-up enable/disable.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * -		PORT_NO_P3 : port 3
 * -		PORT_NO_P4 : port 4
 * @param	selectBit; unsigned char:		select pull-up enable / disable
 * -		PORT_PX0_DISABLE : bit 0 disable
 * -		PORT_PX0_ENABLE  : bit 0 enable
 * -		PORT_PX1_DISABLE : bit 1 disable
 * -		PORT_PX1_ENABLE  : bit 1 enable
 * -		PORT_PX2_DISABLE : bit 2 disable
 * -		PORT_PX2_ENABLE  : bit 2 enable
 * -		PORT_PX3_DISABLE : bit 3 disable
 * -		PORT_PX3_ENABLE  : bit 3 enable
 * -		PORT_PX4_DISABLE : bit 4 disable
 * -		PORT_PX4_ENABLE  : bit 4 enable
 * -		PORT_PX5_DISABLE : bit 5 disable
 * -		PORT_PX5_ENABLE  : bit 5 enable
 * -		PORT_PX6_DISABLE : bit 6 disable
 * -		PORT_PX6_ENABLE  : bit 6 enable
 * -		PORT_PX7_DISABLE : bit 7 disable
 * -		PORT_PX7_ENABLE  : bit 7 enable
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_controlPullup(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo >= PORT_NO_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reserve bit mask
	selectBit &= PORT_RSVMSK_DATA[portNo];
	
	// set pull-up enable
	RegWrite(R08_PX_PU(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports schmitt trigger
 * - 1. check parameters.
 * - 2. set a schmitt trigger enable/disable.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * -		PORT_NO_P2 : port 2
 * @param	selectBit; unsigned char:		select schmitt trigger enable / disable
 * -		PORT_PX0_DISABLE : bit 0 disable
 * -		PORT_PX0_ENABLE  : bit 0 enable
 * -		PORT_PX1_DISABLE : bit 1 disable
 * -		PORT_PX1_ENABLE  : bit 1 enable
 * -		PORT_PX2_DISABLE : bit 2 disable
 * -		PORT_PX2_ENABLE  : bit 2 enable
 * -		PORT_PX3_DISABLE : bit 3 disable
 * -		PORT_PX3_ENABLE  : bit 3 enable
 * -		PORT_PX4_DISABLE : bit 4 disable
 * -		PORT_PX4_ENABLE  : bit 4 enable
 * -		PORT_PX5_DISABLE : bit 5 disable
 * -		PORT_PX5_ENABLE  : bit 5 enable
 * -		PORT_PX6_DISABLE : bit 6 disable
 * -		PORT_PX6_ENABLE  : bit 6 enable
 * -		PORT_PX7_DISABLE : bit 7 disable
 * -		PORT_PX7_ENABLE  : bit 7 enable
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_controlSchmittTrigger(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo > PORT_SMT_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reserve bit mask
	selectBit &= PORT_SMTMSK_DATA[portNo];
	
	// set schmitt trigger input enable
	RegWrite(R08_PX_SM(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * @param	intLevel; unsigned char:		interrupt level
 * -		PORT_INT_LEVEL0 : level0
 * -		PORT_INT_LEVEL1 : level1
 * -		PORT_INT_LEVEL2 : level2
 * -		PORT_INT_LEVEL3 : level3
 * -		PORT_INT_LEVEL4 : level4
 * -		PORT_INT_LEVEL5 : level5
 * -		PORT_INT_LEVEL6 : level6
 * -		PORT_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_initInt(unsigned char portNo, unsigned char intLevel)
{
	// parameter check
	if( (portNo > PORT_INT_MAX) ||
		(intLevel >= PORT_INT_LEVELMAX ) ) {
			return	PORT_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(PORT_INTLVL_REG[portNo], intLevel);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		PORT_PX0_DISABLE : bit 0 interrupt disable
 * -		PORT_PX0_ENABLE  : bit 0 interrupt enable
 * -		PORT_PX1_DISABLE : bit 1 interrupt disable
 * -		PORT_PX1_ENABLE  : bit 1 interrupt enable
 * -		PORT_PX2_DISABLE : bit 2 interrupt disable
 * -		PORT_PX2_ENABLE  : bit 2 interrupt enable
 * -		PORT_PX3_DISABLE : bit 3 interrupt disable
 * -		PORT_PX3_ENABLE  : bit 3 interrupt enable
 * -		PORT_PX4_DISABLE : bit 4 interrupt disable
 * -		PORT_PX4_ENABLE  : bit 4 interrupt enable
 * -		PORT_PX5_DISABLE : bit 5 interrupt disable
 * -		PORT_PX5_ENABLE  : bit 5 interrupt enable
 * -		PORT_PX6_DISABLE : bit 6 interrupt disable
 * -		PORT_PX6_ENABLE  : bit 6 interrupt enable
 * -		PORT_PX7_DISABLE : bit 7 interrupt disable
 * -		PORT_PX7_ENABLE  : bit 7 interrupt enable
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_controlInt(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo > PORT_INT_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	PORT_resetIntFlag(portNo, selectBit);
	
	// set interrupt enable
	RegWrite(R08_PX_IMSK(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports interrupt edge select.
 * - 1. check parameters.
 * - 2. set an interrupt rising-edge/falling-edge.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * @param	selectBit; unsigned char:		select interrupt edge
 * -		PORT_PX0_DISABLE : bit 0 interrupt rising-edge
 * -		PORT_PX0_ENABLE  : bit 0 interrupt falling-edge
 * -		PORT_PX1_DISABLE : bit 1 interrupt rising-edge
 * -		PORT_PX1_ENABLE  : bit 1 interrupt falling-edge
 * -		PORT_PX2_DISABLE : bit 2 interrupt rising-edge
 * -		PORT_PX2_ENABLE  : bit 2 interrupt falling-edge
 * -		PORT_PX3_DISABLE : bit 3 interrupt rising-edge
 * -		PORT_PX3_ENABLE  : bit 3 interrupt falling-edge
 * -		PORT_PX4_DISABLE : bit 4 interrupt rising-edge
 * -		PORT_PX4_ENABLE  : bit 4 interrupt falling-edge
 * -		PORT_PX5_DISABLE : bit 5 interrupt rising-edge
 * -		PORT_PX5_ENABLE  : bit 5 interrupt falling-edge
 * -		PORT_PX6_DISABLE : bit 6 interrupt rising-edge
 * -		PORT_PX6_ENABLE  : bit 6 interrupt falling-edge
 * -		PORT_PX7_DISABLE : bit 7 interrupt rising-edge
 * -		PORT_PX7_ENABLE  : bit 7 interrupt falling-edge
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_setIntEdge(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo > PORT_INT_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// set interrupt edge
	RegWrite(R08_PX_EDGE(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * @param	selectBit; unsigned char:		reset interrupt status
 * -		PORT_PX0_RESET : bit 0 interrupt status reset
 * -		PORT_PX1_RESET : bit 1 interrupt status reset
 * -		PORT_PX2_RESET : bit 2 interrupt status reset
 * -		PORT_PX3_RESET : bit 3 interrupt status reset
 * -		PORT_PX4_RESET : bit 4 interrupt status reset
 * -		PORT_PX5_RESET : bit 5 interrupt status reset
 * -		PORT_PX6_RESET : bit 6 interrupt status reset
 * -		PORT_PX7_RESET : bit 7 interrupt status reset
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_resetIntFlag(unsigned char portNo, unsigned char selectBit)
{
	// parameter check
	if(portNo > PORT_INT_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegWrite(R08_PX_IFLG(portNo), selectBit);
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_NO_P0 : port 0
 * -		PORT_NO_P1 : port 1
 * @param	*flag; unsigned char:			check interrupt flag
 * -		PORT_PX0_NOTOCCURRED : bit 0 interrupt is not occurred
 * -		PORT_PX0_OCCURRED    : bit 0 interrupt is occurred
 * -		PORT_PX1_NOTOCCURRED : bit 1 interrupt is not occurred
 * -		PORT_PX1_OCCURRED    : bit 1 interrupt is occurred
 * -		PORT_PX2_NOTOCCURRED : bit 2 interrupt is not occurred
 * -		PORT_PX2_OCCURRED    : bit 2 interrupt is occurred
 * -		PORT_PX3_NOTOCCURRED : bit 3 interrupt is not occurred
 * -		PORT_PX3_OCCURRED    : bit 3 interrupt is occurred
 * -		PORT_PX4_NOTOCCURRED : bit 4 interrupt is not occurred
 * -		PORT_PX4_OCCURRED    : bit 4 interrupt is occurred
 * -		PORT_PX5_NOTOCCURRED : bit 5 interrupt is not occurred
 * -		PORT_PX5_OCCURRED    : bit 5 interrupt is occurred
 * -		PORT_PX6_NOTOCCURRED : bit 6 interrupt is not occurred
 * -		PORT_PX6_OCCURRED    : bit 6 interrupt is occurred
 * -		PORT_PX7_NOTOCCURRED : bit 7 interrupt is not occurred
 * -		PORT_PX7_OCCURRED    : bit 7 interrupt is occurred
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_checkIntFlag(unsigned char portNo, unsigned char *flag)
{
	// parameter check
	if(portNo > PORT_INT_MAX) {
		return	PORT_INVALID_PARAMETER;
	}
	
	// get interrupt flag
	*flag = RegRead(R08_PX_IFLG(portNo));
	
	return	PORT_SUCCESS;
}

/**
 * @brief	Px ports chattering removal.
 * - 1. check parameters.
 * - 2. disable interrupt of select port number.
 * - 3. set a chattering removal.
 *
 * @param	portNo; unsigned char: 			port number
 * -		PORT_CHATNO_P030 : bit 3-0 of port 0
 * -		PORT_CHATNO_P074 : bit 7-4 of port 0
 * -		PORT_CHATNO_P130 : bit 3-0 of port 1
 * -		PORT_CHATNO_P174 : bit 7-4 of port 1
 * @param	selectBit; unsigned char:		chattering time
 * -		PORT_CHATTIME_NONE  :   none
 * -		PORT_CHATTIME_256   :   256/fPCLK
 * -		PORT_CHATTIME_512   :   512/fPCLK
 * -		PORT_CHATTIME_1024  :  1024/fPCLK
 * -		PORT_CHATTIME_2048  :  2048/fPCLK
 * -		PORT_CHATTIME_4096  :  4096/fPCLK
 * -		PORT_CHATTIME_8192  :  8192/fPCLK
 * -		PORT_CHATTIME_16384 : 16384/fPCLK
 * @return	short:							end status
 * -		PORT_SUCCESS : success
 * -		PORT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PORT_setChatteringFilter(unsigned char portNo, unsigned char selectBit)
{
	unsigned char	mask;
	unsigned char	port = portNo >> 1;										// get P0 or P1

	// parameter check
	if(	(portNo >= PORT_CHAT_MAX) ||
		(selectBit >= PORT_CHATTIME_MAX) ) {
			return	PORT_INVALID_PARAMETER;
	}
	
	// interrupt disable
	PORT_controlInt(port, (PORT_PX7_DISABLE | PORT_PX6_DISABLE | PORT_PX5_DISABLE |
		PORT_PX4_DISABLE | PORT_PX3_DISABLE | PORT_PX2_DISABLE | PORT_PX1_DISABLE | PORT_PX0_DISABLE));
	
	// set chattering filter time
	if((portNo & PORT_CHAT_LH) == PORT_CHAT_LH) {							// Px[7:4]
		mask = CHAT_PXCF2;
		selectBit <<= PXCF2_SFT;											// move bit position
	} else {																// Px[3:0]
		mask = CHAT_PXCF1;
	}
	RegModify(R08_PX_CHAT(port), mask, selectBit);
	
	return	PORT_SUCCESS;
}

#endif	// end of PE_PORT
