/**
 * @file	remc.c
 * @brief	REMC driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_REMC
#include <remc_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */
#define	REMC_CARRIER_MAX					0x003fU							// carrier length max
#define	RX_DATA_LENGTH_COUNT				0x00ffU							// receive data length count


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short REMC_init(REMC_INIT_PARAM *param);
short REMC_setEnable(unsigned short selectMode, unsigned short selectBit);
short REMC_setTransmitData(unsigned short data, unsigned short length);
void REMC_setReceiveLength(void);
void REMC_getReceiveData(unsigned short *data);
void REMC_calcReceiveDataPulse(unsigned short *data);
short REMC_initInt(unsigned char intLevel);
short REMC_controlInt(unsigned short selectBit);
short REMC_resetIntFlag(unsigned short selectBit);
void REMC_checkIntFlag(unsigned short *flag);


/**
 * @brief	REMC initialize.
 * - 1. check parameters.
 * - 2. stop a TX/RX.
 * - 3. set REMC ports.
 * - 4. set a carrier generator clock.
 * - 5. set a length counter clock.
 * - 6. set a L carrier length.
 * - 7. set a H carrier length.
 *
 * @param	*param; REMC_INIT_PARAM:		REMC initialize parameter
 * -		selectMode; unsigned short:		select transfer direction
 * 	-			REMC_TRANSMIT : transmit mode
 * 	-			REMC_RECEIVE  : receive mode
 * -		selectCarrier; unsigned short:	select carrier generator clock
 * 	-			REMC_CLK1     : PCLK 1/1
 * 	-			REMC_CLK2     : PCLK 1/2
 * 	-			REMC_CLK4     : PCLK 1/4
 * 	-			REMC_CLK8     : PCLK 1/8
 * 	-			REMC_CLK16    : PCLK 1/16
 * 	-			REMC_CLK32    : PCLK 1/32
 * 	-			REMC_CLK64    : PCLK 1/64
 * 	-			REMC_CLK128   : PCLK 1/128
 * 	-			REMC_CLK256   : PCLK 1/256
 * 	-			REMC_CLK512   : PCLK 1/512
 * 	-			REMC_CLK1024  : PCLK 1/1024
 * 	-			REMC_CLK2048  : PCLK 1/2048
 * 	-			REMC_CLK4096  : PCLK 1/4096
 * 	-			REMC_CLK8192  : PCLK 1/8192
 * 	-			REMC_CLK16384 : PCLK 1/16384
 * -		selectLength; unsigned short:	select length counter clock
 * 	-			REMC_CLK1     : PCLK 1/1
 * 	-			REMC_CLK2     : PCLK 1/2
 * 	-			REMC_CLK4     : PCLK 1/4
 * 	-			REMC_CLK8     : PCLK 1/8
 * 	-			REMC_CLK16    : PCLK 1/16
 * 	-			REMC_CLK32    : PCLK 1/32
 * 	-			REMC_CLK64    : PCLK 1/64
 * 	-			REMC_CLK128   : PCLK 1/128
 * 	-			REMC_CLK256   : PCLK 1/256
 * 	-			REMC_CLK512   : PCLK 1/512
 * 	-			REMC_CLK1024  : PCLK 1/1024
 * 	-			REMC_CLK2048  : PCLK 1/2048
 * 	-			REMC_CLK4096  : PCLK 1/4096
 * 	-			REMC_CLK8192  : PCLK 1/8192
 * 	-			REMC_CLK16384 : PCLK 1/16384
 * -		carrireL; unsigned short:		select L carrier length
 * 	-			from 0x0000 to 0x003f
 * -		carrireH; unsigned short:		select H carrier length
 * 	-			from 0x0000 to 0x003f
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_init(REMC_INIT_PARAM *param)
{
	// parameter check
	if(	(param->selectMode > REMC_RECEIVE) ||
		(param->selectCarrier >= REMC_CLK_MAX) ||
		(param->selectLength >= REMC_CLK_MAX) ||
		(param->carrireL > REMC_CARRIER_MAX) ||
		(param->carrireH > REMC_CARRIER_MAX) ) {
			return	REMC_INVALID_PARAMETER;
	}
	
	// REMC TX / RX disable
	REMC_setEnable(param->selectMode, REMC_DISABLE);
	
	// REMC port set
	MUX_setREMCport(MUX_ON);
	
	// Carrier Generator / Length Counter Clock set
	RegModify(R16_REMC_CFG, (CGCLK | LCCLK),
		((param->selectCarrier << CGCLK_SFT) | (param->selectLength << LCCLK_SFT)));
	
	// Carrier Length set
	RegWrite(R16_REMC_CAR, ((param->carrireL << REMCL_SFT) | (param->carrireH << REMCH_SFT)));
	
	return	REMC_SUCCESS;
}

/**
 * @brief	set a REMC transfer enable / disable.
 * - 1. check parameters.
 * - 2. set a 0xff to the receive data length count.
 * - 3. set a transfer direction.
 * - 4. set a REMC transfer enable/disable.
 *
 * @param	selectMode; unsigned short: 	select transfer mode
 * -		REMC_TRANSMIT : transmit mode
 * -		REMC_RECEIVE  : receive mode
 * @param	selectBit; unsigned short:		select enable / disable
 * -		REMC_DISABLE : transfer disable
 * -		REMC_ENABLE  : transfer enable
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_setEnable(unsigned short selectMode, unsigned short selectBit)
{
	// parameter check
	if(	(selectMode > REMC_RECEIVE) ||
		(selectBit > REMC_ENABLE) ) {
			return	REMC_INVALID_PARAMETER;
	}
	
	// receive data length count set
	REMC_setReceiveLength();
	
	// REMC mode & enable / disable set
	RegModify(R16_REMC_CFG, (REMMD | REMEN), ((selectMode << REMMD_SFT) | (selectBit << REMEN_SFT)));
	
	return	REMC_SUCCESS;
}

/**
 * @brief	set a transmit data.
 * - 1. check parameters.
 * - 2. set a transmit data.
 * - 3. set a transmit data length count.
 *
 * @param	data; unsigned short: 			transmit data
 * -		REMC_DATA_L : 0(L)
 * -		REMC_DATA_H : 1(H)
 * @param	length; unsigned short:			transmit data length count
 * -		from 0x0000 to 0x00ff
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_setTransmitData(unsigned short data, unsigned short length)
{
	// parameter check
	if(	(data > REMC_DATA_H) ||
		(length > REMC_LENGTH_MAX) ) {
			return	REMC_INVALID_PARAMETER;
	}
	
	// transmit data & data length count set
	RegWrite(R16_REMC_LCNT, ((length << REMLEN_SFT) | (data << REMDT_SFT)));
	
	return	REMC_SUCCESS;
}

/**
 * @brief	set a receive data length count.
 * - 1. set a 0xff to the receive data length count.
 *
 * @return	none
 * @date	2009/12/25
 */
void REMC_setReceiveLength(void)
{
	// receive data length count set
	RegModify(R16_REMC_LCNT, REMLEN, (RX_DATA_LENGTH_COUNT << REMLEN_SFT));
}

/**
 * @brief	get a receive data.
 * - 1. get a receive data and set to "data".
 *
 * @param	*data; unsigned short:			receive data
 * -		REMC_DATA_L : receive data 0(L)
 * -		REMC_DATA_H : receive data 1(H)
 * @return	none
 * @date	2009/12/25
 */
void REMC_getReceiveData(unsigned short *data)
{
	// receive data get
	*data = (RegRead(R16_REMC_LCNT) & REMDT) >> REMDT_SFT;
}

/**
 * @brief	calculate a receive data pulse.
 * - 1. get a receive data length count and calculate a pulse length and set to "data".
 *
 * @param	*data; unsigned short:			receive data pulse
 * -		from 0x0000 to 0x00ff
 * @return	none
 * @date	2009/12/25
 */
void REMC_calcReceiveDataPulse(unsigned short *data)
{
	unsigned short	count;													// receive data length count
	
	// receive length count get
	count = (RegRead(R16_REMC_LCNT) & REMLEN) >> REMLEN_SFT;
	
	// receive data pulse calculate
	*data = RX_DATA_LENGTH_COUNT - count;
}

/**
 * @brief	REMC interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		REMC_INT_LEVEL0 : level0
 * -		REMC_INT_LEVEL1 : level1
 * -		REMC_INT_LEVEL2 : level2
 * -		REMC_INT_LEVEL3 : level3
 * -		REMC_INT_LEVEL4 : level4
 * -		REMC_INT_LEVEL5 : level5
 * -		REMC_INT_LEVEL6 : level6
 * -		REMC_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= REMC_INT_LEVELMAX) {
		return	REMC_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(REMC_INTLVL, intLevel);
	
	return	REMC_SUCCESS;
}

/**
 * @brief	REMC interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		REMC_UDFINT_DISABLE     : underflow interrupt disable
 * -		REMC_UDFINT_ENABLE      : underflow interrupt enable
 * -		REMC_RISINGINT_DISABLE  : rising edge interrupt disable
 * -		REMC_RISINGINT_ENABLE   : rising edge interrupt enable
 * -		REMC_FALLINGINT_DISABLE : falling edge interrupt disable
 * -		REMC_FALLINGINT_ENABLE  : falling edge interrupt enable
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_controlInt(unsigned short selectBit)
{
	// parameter check
	if(selectBit > REMC_IEN_MASK) {
		return	REMC_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	REMC_resetIntFlag(selectBit << (REMUIF_SFT - REMUIE_SFT));
	
	// set interrupt enable / disable
	RegModify(R16_REMC_INT, REMC_IEN_MASK, (selectBit << REMUIE_SFT));
	
	return	REMC_SUCCESS;
}

/**
 * @brief	REMC interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	selectBit; unsigned short:		reset interrupt flag
 * -		REMC_UDFINT_RESET     : underflow interrupt flag reset
 * -		REMC_RISINGINT_RESET  : rising edge interrupt flag reset
 * -		REMC_FALLINGINT_RESET : falling edge interrupt flag reset
 * @return	short:							end status
 * -		REMC_SUCCESS : success
 * -		REMC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short REMC_resetIntFlag(unsigned short selectBit)
{
	// parameter check
	if((selectBit & ~REMC_FLG_MASK) != 0) {
		return	REMC_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegModify(R16_REMC_INT, REMC_FLG_MASK, selectBit);
	
	return	REMC_SUCCESS;
}

/**
 * @brief	REMC interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned short:			check interrupt flag
 * -		REMC_UDFINT_NOTOCCURRED     : underflow interrupt is not occurred
 * -		REMC_UDFINT_OCCURRED        : underflow interrupt is occurred
 * -		REMC_RISINGINT_NOTOCCURRED  : rising edge interrupt is not occurred
 * -		REMC_RISINGINT_OCCURRED     : rising edge interrupt is occurred
 * -		REMC_FALLINGINT_NOTOCCURRED : falling edge interrupt is not occurred
 * -		REMC_FALLINGINT_OCCURRED    : falling edge interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void REMC_checkIntFlag(unsigned short *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_REMC_INT) & REMC_FLG_MASK;
}

#endif	// end of PE_REMC
