/**
 * @file	rfc.c
 * @brief	RFC driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_RFC
#include <rfc_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short RFC_setRFC(unsigned char selectBit);
short RFC_setRFCChannel(unsigned char selectBit);
short RFC_setRFCMode(unsigned char selectBit);
short RFC_setReferenceOscillation(unsigned char selectBit);
short RFC_setSensorAOscillation(unsigned char selectBit);
short RFC_setSensorBOscillation(unsigned char selectBit);
void RFC_getReferenceOscillation(unsigned char *condition);
void RFC_getSensorAOscillation(unsigned char *condition);
void RFC_getSensorBOscillation(unsigned char *condition);
short RFC_setEventMode(unsigned char selectBit);
short RFC_setContinuous(unsigned char selectBit);
short RFC_setMeasurementCounter(unsigned long counter);
short RFC_setTimeBaseCounter(unsigned long counter);
void RFC_getMeasurementCounter(unsigned long *counter);
void RFC_getTimeBaseCounter(unsigned long *counter);
short RFC_initInt(unsigned char intLevel);
short RFC_controlInt(unsigned char selectBit);
short RFC_resetIntFlag(unsigned char selectBit);
void RFC_checkIntFlag(unsigned char *flag);


/**
 * @brief	set a RFC enable / disable.
 * - 1. check parameters.
 * - 2. set a RFC enable / disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		RFC_DISABLE : RFC disable
 * -		RFC_ENABLE  : RFC enable
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setRFC(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_ENABLE) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// RFC enable / disable
	RegModify(R16_RFC_CTL, RFCEN, (selectBit << RFCEN_SFT));
	
	return	RFC_SUCCESS;
}

#ifdef	S1C17602
/**
 * @brief	set a conversion channel.
 * - 1. check parameters.
 * - 2. set a conversion channel.
 *
 * @param	selectBit; unsigned char: 		select conversion channel
 * -		RFC_CONVERSION_CH0 : channel 0 conversion
 * -		RFC_CONVERSION_CH1 : channel 1 conversion
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setRFCChannel(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= RFC_CH_MAX) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// conversion channel set
	RegModify(R16_RFC_CTL, CHSEL, (selectBit << CHSEL_SFT));
	
	return	RFC_SUCCESS;
}
#endif		// end of S1C17602

/**
 * @brief	set a sensor oscillation mode.
 * - 1. check parameters.
 * - 2. set a sensor oscillation mode.
 *
 * @param	selectBit; unsigned char: 		select oscillation mode
 * -		RFC_OSCMODE_RDC : resistive sensor DC oscillation mode
 * -		RFC_OSCMODE_RAC : resistive sensor AC oscillation mode
 * -		RFC_OSCMODE_CDC : capacitive sensor DC oscillation mode
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setRFCMode(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= RFC_OSCMODE_MAX) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// sensor oscillation mode set
	RegModify(R16_RFC_CTL, SMODE, (selectBit << SMODE_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a reference oscillation start / stop.
 * - 1. check parameters.
 * - 2. set a reference oscillation start / stop.
 *
 * @param	selectBit; unsigned char: 		select start / stop
 * -		RFC_REFOSC_STOP  : reference oscillation stop
 * -		RFC_REFOSC_START : reference oscillation start
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setReferenceOscillation(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_REFOSC_START) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// reference oscillation start / stop
	RegModify(R16_RFC_TRG, SREF, (selectBit << SREF_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a sensor A oscillation start / stop.
 * - 1. check parameters.
 * - 2. set a sensor A oscillation start / stop.
 *
 * @param	selectBit; unsigned char: 		select start / stop
 * -		RFC_SENAOSC_STOP  : sensor A oscillation stop
 * -		RFC_SENAOSC_START : sensor A oscillation start
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setSensorAOscillation(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_SENAOSC_START) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// sensor A oscillation start / stop
	RegModify(R16_RFC_TRG, SSENA, (selectBit << SSENA_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a sensor B oscillation start / stop.
 * - 1. check parameters.
 * - 2. set a sensor B oscillation start / stop.
 *
 * @param	selectBit; unsigned char: 		select start / stop
 * -		RFC_SENBOSC_STOP  : sensor B oscillation stop
 * -		RFC_SENBOSC_START : sensor B oscillation start
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setSensorBOscillation(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_SENBOSC_START) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// sensor B oscillation start / stop
	RegModify(R16_RFC_TRG, SSENB, (selectBit << SSENB_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	get a reference oscillation.
 * - 1. get a reference oscillation condition and set to "condition".
 *
 * @param	*condition; unsigned char: 		oscillation status
 * -		RFC_REFOSC_STOP : reference oscillation stop
 * -		RFC_REFOSC_RUN  : reference oscillation run
 * @return	none
 * @date	2009/12/25
 */
void RFC_getReferenceOscillation(unsigned char *condition)
{
	// reference oscillation get
	*condition = (RegRead(R16_RFC_TRG) & SREF) >> SREF_SFT;
}

/**
 * @brief	get a sensor A oscillation.
 * - 1. get a sensor A oscillation condition and set to "condition".
 *
 * @param	*condition; unsigned char: 		oscillation status
 * -		RFC_SENAOSC_STOP : sensor A oscillation stop
 * -		RFC_SENAOSC_RUN  : sensor A oscillation run
 * @return	none
 * @date	2009/12/25
 */
void RFC_getSensorAOscillation(unsigned char *condition)
{
	// sensor A oscillation get
	*condition = (RegRead(R16_RFC_TRG) & SSENA) >> SSENA_SFT;
}

/**
 * @brief	get a sensor B oscillation.
 * - 1. get a sensor B oscillation condition and set to "condition".
 *
 * @param	*condition; unsigned char: 		oscillation status
 * -		RFC_SENBOSC_STOP : sensor B oscillation stop
 * -		RFC_SENBOSC_RUN  : sensor B oscillation run
 * @return	none
 * @date	2009/12/25
 */
void RFC_getSensorBOscillation(unsigned char *condition)
{
	// sensor B oscillation get
	*condition = (RegRead(R16_RFC_TRG) & SSENB) >> SSENB_SFT;
}

/**
 * @brief	set an event counter mode enable / disable.
 * - 1. check parameters.
 * - 2. set an event counter mode enable / disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		RFC_EVTMODE_DISABLE : normal mode
 * -		RFC_EVTMODE_ENABLE  : external clock input mode(event counter mode)
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setEventMode(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_EVTMODE_ENABLE) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// event counter mode enable / disable
	RegModify(R16_RFC_CTL, EVTEN, (selectBit << EVTEN_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a continuous oscillation enable / disable.
 * - 1. check parameters.
 * - 2. set a continuous oscillation enable / disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		RFC_CONTINUOUS_DISABLE : continuous oscillation disable
 * -		RFC_CONTINUOUS_ENABLE  : continuous oscillation enable
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setContinuous(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_CONTINUOUS_ENABLE) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// continuous oscillation enable / disable
	RegModify(R16_RFC_CTL, CONEN, (selectBit << CONEN_SFT));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a measurement counter.
 * - 1. check parameters.
 * - 2. set a measurement counter value.
 *
 * @param	counter; unsigned long: 		measurement counter value
 * -		from 0x00000000 to 0x00ffffff
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setMeasurementCounter(unsigned long counter)
{
	// parameter check
	if((counter >> 16) > MC23_16) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// measurement counter set
	RegWrite(R16_RFC_MCL, (unsigned short)(counter & 0x0000ffff));
	RegWrite(R16_RFC_MCH, (unsigned short)((counter >> 16) & 0x0000ffff));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	set a timer base counter.
 * - 1. check parameters.
 * - 2. set a timer base counter value.
 *
 * @param	counter; unsigned long: 		timer base counter value
 * -		from 0x00000000 to 0x00ffffff
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_setTimeBaseCounter(unsigned long counter)
{
	// parameter check
	if((counter >> 16) > TC23_16) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// timer base counter set
	RegWrite(R16_RFC_TCL, (unsigned short)(counter & 0x0000ffff));
	RegWrite(R16_RFC_TCH, (unsigned short)((counter >> 16) & 0x0000ffff));
	
	return	RFC_SUCCESS;
}

/**
 * @brief	get a measurement counter.
 * - 1. read a measurement counter value and set to "counter".
 *
 * @param	*counter; unsigned long: 		measurement counter value
 * -		from 0x00000000 to 0x00ffffff
 * @return	none
 * @date	2009/12/25
 */
void RFC_getMeasurementCounter(unsigned long *counter)
{
	// measurement counter get
	*counter   = (unsigned long)RegRead(R16_RFC_MCH);
	*counter <<= 16;
	*counter  |= (unsigned long)RegRead(R16_RFC_MCL);
}

/**
 * @brief	get a timer base counter.
 * - 1. read a timer base counter value and set to "counter".
 *
 * @param	*counter; unsigned long: 		timer base counter value
 * -		from 0x00000000 to 0x00ffffff
 * @return	none
 * @date	2009/12/25
 */
void RFC_getTimeBaseCounter(unsigned long *counter)
{
	// timer base counter get
	*counter   = (unsigned long)RegRead(R16_RFC_TCH);
	*counter <<= 16;
	*counter  |= (unsigned long)RegRead(R16_RFC_TCL);
}

/**
 * @brief	RFC interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		RFC_INT_LEVEL0 : level0
 * -		RFC_INT_LEVEL1 : level1
 * -		RFC_INT_LEVEL2 : level2
 * -		RFC_INT_LEVEL3 : level3
 * -		RFC_INT_LEVEL4 : level4
 * -		RFC_INT_LEVEL5 : level5
 * -		RFC_INT_LEVEL6 : level6
 * -		RFC_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= RFC_INT_LEVELMAX) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(RFC_INTLVL, intLevel);
	
	return	RFC_SUCCESS;
}

/**
 * @brief	RFC interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		RFC_TCOVFINT_DISABLE : time base counter over flow error interrupt disable
 * -		RFC_TCOVFINT_ENABLE  : time base counter over flow error interrupt enable
 * -		RFC_MCOVFINT_DISABLE : measurement counter over flow error interrupt disable
 * -		RFC_MCOVFINT_ENABLE  : measurement counter over flow error interrupt enable
 * -		RFC_SENBINT_DISABLE  : sensor B oscillation end interrupt disable
 * -		RFC_SENBINT_ENABLE   : sensor B oscillation end interrupt enable
 * -		RFC_SENAINT_DISABLE  : sensor A oscillation end interrupt disable
 * -		RFC_SENAINT_ENABLE   : sensor A oscillation end interrupt enable
 * -		RFC_REFINT_DISABLE   : reference oscillation end interrupt disable
 * -		RFC_REFINT_ENABLE    : reference oscillation end interrupt enable
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_controlInt(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_IMSK_MASK) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RFC_resetIntFlag(selectBit);
	
	// set interrupt enable / disable
	RegWrite(R16_RFC_IMSK, selectBit);
	
	return	RFC_SUCCESS;
}

/**
 * @brief	RFC interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	selectBit; unsigned char:		reset interrupt flag
 * -		RFC_TCOVFINT_RESET : time base counter over flow error interrupt flag reset
 * -		RFC_MCOVFINT_RESET : measurement counter over flow error interrupt flag reset
 * -		RFC_SENBINT_RESET  : sensor B oscillation end nterrupt flag reset
 * -		RFC_SENAINT_RESET  : sensor A oscillation end nterrupt flag reset
 * -		RFC_REFINT_RESET   : reference oscillation end interrupt flag reset
 * @return	short:							end status
 * -		RFC_SUCCESS : success
 * -		RFC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short RFC_resetIntFlag(unsigned char selectBit)
{
	// parameter check
	if(selectBit > RFC_IFLG_MASK) {
		return	RFC_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegWrite(R16_RFC_IFLG, selectBit);
	
	return	RFC_SUCCESS;
}

/**
 * @brief	RFC interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned char:			check interrupt flag
 * -		RFC_TCOVFINT_NOTOCCURRED : time base counter over flow error interrupt is not occurred
 * -		RFC_TCOVFINT_OCCURRED    : time base counter over flow error interrupt is occurred
 * -		RFC_MCOVFINT_NOTOCCURRED : measurement counter over flow error interrupt is not occurred
 * -		RFC_MCOVFINT_OCCURRED    : measurement counter over flow error interrupt is occurred
 * -		RFC_SENBINT_NOTOCCURRED  : sensor B oscillation end interrupt is not occurred
 * -		RFC_SENBINT_OCCURRED     : sensor B oscillation end interrupt is occurred
 * -		RFC_SENAINT_NOTOCCURRED  : sensor A oscillation end interrupt is not occurred
 * -		RFC_SENAINT_OCCURRED     : sensor A oscillation end interrupt is occurred
 * -		RFC_REFINT_NOTOCCURRED   : reference oscillation end interrupt is not occurred
 * -		RFC_REFINT_OCCURRED      : reference oscillation end interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void RFC_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R16_RFC_IFLG) & RFC_IFLG_MASK;
}

#endif	// end of PE_RFC
