/**
 * @file	svd.c
 * @brief	SVD driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_SVD
#include <svd_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short SVD_setCompareVoltage(unsigned char selectBit);
short SVD_controlDetection(unsigned char selectBit);
void SVD_getDetectionResult(unsigned char *result);
short SVD_initInt(unsigned char intLevel);
short SVD_controlInt(unsigned char selectBit);
void SVD_resetIntFlag(void);
void SVD_checkIntFlag(unsigned char *flag);


/**
 * @brief	set a SVD compare voltage.
 * - 1. check parameters.
 * - 2. set a compare voltage for detection voltage dips.
 *
 * @param	selectBit; unsigned char: 		select compare voltage
 * -		SVD_COMPVOL_18V : 1.8V
 * -		SVD_COMPVOL_19V : 1.9V
 * -		SVD_COMPVOL_20V : 2.0V
 * -		SVD_COMPVOL_21V : 2.1V
 * -		SVD_COMPVOL_22V : 2.2V
 * -		SVD_COMPVOL_23V : 2.3V
 * -		SVD_COMPVOL_24V : 2.4V
 * -		SVD_COMPVOL_25V : 2.5V
 * -		SVD_COMPVOL_26V : 2.6V
 * -		SVD_COMPVOL_27V : 2.7V
 * -		SVD_COMPVOL_28V : 2.8V
 * -		SVD_COMPVOL_29V : 2.9V
 * -		SVD_COMPVOL_30V : 3.0V
 * -		SVD_COMPVOL_31V : 3.1V
 * -		SVD_COMPVOL_32V : 3.2V
 * @return	short:							end status
 * -		SVDT_SUCCESS : success
 * -		SVDT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SVD_setCompareVoltage(unsigned char selectBit)
{
	// parameter check
	if(	(selectBit == SVD_COMPVOL_RSV) ||
		(selectBit >= SVD_COMPVOL_MAX) ) {
			return	SVD_INVALID_PARAMETER;
	}
	
	// SVD compare voltage set
	RegWrite(R08_SVD_CMP, (selectBit << SVDC_SFT));
	
	return	SVD_SUCCESS;
}

/**
 * @brief	set a SVD detection enable / disable.
 * - 1. check parameters.
 * - 2. set a detection of power supply voltage enable / disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		SVD_DETECT_DISABLE : detection of power supply voltage disable
 * -		SVD_DETECT_ENABLE  : detection of power supply voltage enable
 * @return	short:							end status
 * -		SVD_SUCCESS : success
 * -		SVD_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SVD_controlDetection(unsigned char selectBit)
{
	// parameter check
	if(selectBit > SVD_DETECT_ENABLE) {
		return	SVD_INVALID_PARAMETER;
	}
	
	// SVD detection enable / disable
	RegWrite(R08_SVD_EN, (selectBit << SVDEN_SFT));
	
	return	SVD_SUCCESS;
}

/**
 * @brief	get a SVD detection result.
 * - 1. get a detection result and set to "result".
 *
 * @param	*result; unsigned char: 		detection result
 * -		SVD_COMPVOL_NORMAL : power supply voltage(VDD) >= compare voltage
 * -		SVD_COMPVOL_LOW    : power supply voltage(VDD) <  compare voltage
 * @return	none
 * @date	2009/12/25
 */
void SVD_getDetectionResult(unsigned char *result)
{
	// SVD detection result get
	*result = RegRead(R08_SVD_RSLT) & SVDDT;
}

/**
 * @brief	SVD interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		SVD_INT_LEVEL0 : level0
 * -		SVD_INT_LEVEL1 : level1
 * -		SVD_INT_LEVEL2 : level2
 * -		SVD_INT_LEVEL3 : level3
 * -		SVD_INT_LEVEL4 : level4
 * -		SVD_INT_LEVEL5 : level5
 * -		SVD_INT_LEVEL6 : level6
 * -		SVD_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		SVD_SUCCESS : success
 * -		SVD_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SVD_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= SVD_INT_LEVELMAX) {
		return	SVD_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(SVD_INTLVL, intLevel);
	
	return	SVD_SUCCESS;
}

/**
 * @brief	SVD interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		SVD_INT_DISABLE : SVD interrupt disable
 * -		SVD_INT_ENABLE  : SVD interrupt enable
 * @return	short:							end status
 * -		SVD_SUCCESS : success
 * -		SVD_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SVD_controlInt(unsigned char selectBit)
{
	// parameter check
	if(selectBit > SVD_INT_ENABLE) {
		return	SVD_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	SVD_resetIntFlag();
	
	// set interrupt enable / disable
	RegWrite(R08_SVD_IMSK, selectBit);
	
	return	SVD_SUCCESS;
}

/**
 * @brief	SVD interrupt flag reset.
 * - 1. reset an interrupt status.
 *
 * @return	none
 * @date	2009/12/25
 */
void SVD_resetIntFlag(void)
{
	// reset interrupt flag
	RegWrite(R08_SVD_IFLG, SVD_INT_RESET);
}

/**
 * @brief	SVD interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 *
 * @param	*flag; unsigned char:			check interrupt flag
 * -		SVD_INT_NOTOCCURRED : SVD interrupt is not occurred
 * -		SVD_INT_OCCURRED    : SVD interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void SVD_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_SVD_IFLG) & SVDIF;
}

#endif	// end of PE_SVD
