/**
 * @file	t16e.c
 * @brief	PWM timer driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_T16E
#include <t16e_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static volatile unsigned char	*T16E_INTLVL_REG[] = {
	T16E_INTLVL_CH0,														// Ch. 0
#ifdef	T16E_CHANNEL2
	T16E_INTLVL_CH1															// Ch. 1
#endif	// end of T16E_CHANNEL2
};

/*
 * Declarations of function prototype
 */
short T16E_init(unsigned char chNo, T16E_TIM_MODE *mode, T16E_INIT_PARAM *param);
short T16E_setCompareData(unsigned char chNo, unsigned short dataA, unsigned short dataB);
short T16E_getCounterData(unsigned char chNo, unsigned short *data);
short T16E_setTimerMode(unsigned char chNo, T16E_TIM_MODE *mode);
short T16E_resetTimer(unsigned char chNo);
short T16E_setTimerRun(unsigned char chNo, unsigned short selectBit);
short T16E_setInputClock(unsigned char chNo, unsigned short selectBit);
short T16E_initInt(unsigned char chNo, unsigned char intLevel);
short T16E_controlInt(unsigned char chNo, unsigned short selectBit);
short T16E_resetIntFlag(unsigned char chNo, unsigned short selectBit);
short T16E_checkIntFlag(unsigned char chNo, unsigned short *flag);


/**
 * @brief	PWM timer initialize.
 * - 1. check parameters.
 * - 2. stop a PWM timer.
 * - 3. set an input clock.
 * - 4. set a count clock.(it is not needed for a external clock mode.)
 * - 5. set an initial output level of clock output.
 * - 6. set a polarity of clock output signals.
 * - 7. set a clock output enable.
 * - 8. set a port function.(it is not needed for a clock output disable.)
 * - 9. set a coutner data.
 * - 10. set a compare data.
 * - 11. set a clock output mode.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	*mode; T16E_TIM_MODE: 			mode set parameter
 * -		refer to T16E_setTimerMode
 * @param	*param; T16E_INIT_PARAM: 		initialize parameter
 * -		dataA; unsigned short:			compare data A
 * 	-			from 0x0000 to 0xFFFF
 * -		dataB; unsigned short:			compare data B
 * 	-			from 0x0000 to 0xFFFF
 * -		selectClk; unsigned short:		select count clock
 * 	-			refer to T16E_setInputClock
 * -		data; unsigned short:			counter initial value
 * 	-			from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_init(unsigned char chNo, T16E_TIM_MODE *mode, T16E_INIT_PARAM *param)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		(mode->initol > T16E_TIMMODE_TOUT3H) ||
		(mode->selfm > T16E_TIMMODE_CLK_FINEMODE) ||
		(mode->cbufen > T16E_TIMMODE_CMP_ENABLE) ||
		(mode->invout > T16E_TIMMODE_INVERSE) ||
		(mode->clksel > T16E_TIMMODE_EXTERNAL) ||
		(mode->outen > T16E_TIMMODE_ENABLE) ||
		(param->selectClk >= T16E_CNTCLK_MAX) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// PWM timer stop
	T16E_setTimerRun(chNo, T16E_TIMER_STOP);

	// PWM timer mode/port set
	T16E_setTimerMode(chNo, mode);

	// PWM timer input clock set
	if(mode->clksel != T16E_TIMMODE_EXTERNAL) {
		T16E_setInputClock(chNo, param->selectClk);
	}

	// PWM timer counter data set
	RegWrite(R16_T16E_TC(chNo), param->data);

	// PWM compare data set
	T16E_setCompareData(chNo, param->dataA, param->dataB);

	return	T16E_SUCCESS;
}

/**
 * @brief	set a PWM timer compare data A/B
 * - 1. check parameters.
 * - 2. set a compare data A.
 * - 3. set a compare data B.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	dataA; unsigned short: 			compare data A
 * -		from 0x0000 to 0xFFFF
 * @param	dataB; unsigned short: 			compare data B
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_setCompareData(unsigned char chNo, unsigned short dataA, unsigned short dataB)
{
	// parameter check
	if(chNo >= T16E_CH_MAX) {
		return	T16E_INVALID_PARAMETER;
	}

	// compare data set
	RegWrite(R16_T16E_CA(chNo), dataA);
	RegWrite(R16_T16E_CB(chNo), dataB);

	return	T16E_SUCCESS;
}

/**
 * @brief	get a counter data.
 * - 1. check parameters.
 * - 2. get a counter data from register and set the data to "data".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	*data; unsigned short:			counter data
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_getCounterData(unsigned char chNo, unsigned short *data)
{
	// parameter check
	if(chNo >= T16E_CH_MAX) {
		return	T16E_INVALID_PARAMETER;
	}

	// counter data get
	*data = RegRead(R16_T16E_TC(chNo));

	return	T16E_SUCCESS;
}

/**
 * @brief	set a PWM timer mode.
 * - 1. check parameters.
 * - 2. set a PWM timer port function.(it is not needed for a clock output disable.)
 * - 3. set a PWM timer mode.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	*mode; T16E_TIM_MODE:			mode set parameter
 * -		initol; unsigned short:			select initial output level of PWM timer output
 * 	-			T16E_TIMMODE_TOUT3L : TOUT3 low/TOUTN3 high
 * 	-			T16E_TIMMODE_TOUT3H : TOUT3 high/TOUTN3 low
 * -		selfm; unsigned short:			select clock output
 * 	-			T16E_TIMMODE_CLK_NORMAL   : normal mode
 * 	-			T16E_TIMMODE_CLK_FINEMODE : fine mode
 * -		cbufen; unsigned short:			select compare data buffer write enable / disable
 * 	-			T16E_TIMMODE_CMP_DISABLE : write disable
 * 	-			T16E_TIMMODE_CMP_ENABLE  : write enable
 * -		invout; unsigned short:			select polarity
 * 	-			T16E_TIMMODE_NORMAL  : normal
 * 	-			T16E_TIMMODE_INVERSE : invert
 * -		clksel; unsigned short:			select input clock
 * 	-			T16E_TIMMODE_INTERNAL : internal clock
 * 	-			T16E_TIMMODE_EXTERNAL : external clock
 * -		outen; unsigned short:			TOUT3/TOUTN3 signals output enable / disable
 * 	-			T16E_TIMMODE_DISABLE : output disable
 * 	-			T16E_TIMMODE_ENABLE  : output enable
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_setTimerMode(unsigned char chNo, T16E_TIM_MODE *mode)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		(mode->initol > T16E_TIMMODE_TOUT3H) ||
		(mode->selfm > T16E_TIMMODE_CLK_FINEMODE) ||
		(mode->cbufen > T16E_TIMMODE_CMP_ENABLE) ||
		(mode->invout > T16E_TIMMODE_INVERSE) ||
		(mode->clksel > T16E_TIMMODE_EXTERNAL) ||
		(mode->outen > T16E_TIMMODE_ENABLE) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// PWM timer port set
	if(mode->outen == T16E_TIMMODE_ENABLE) {
		MUX_setT16Eport(chNo, MUX_ON);
	}

	// PWM timer mode set
	RegModify(R16_T16E_CTL(chNo), T16E_MODE_MASK, ((mode->initol << INITOL_SFT) |
		(mode->selfm << SELFM_SFT) | (mode->cbufen << CBUFEN_SFT) | (mode->invout << INVOUT_SFT) |
		(mode->clksel << CLKSEL_SFT) | (mode->outen << OUTEN_SFT)));

	return	T16E_SUCCESS;
}

/**
 * @brief	reset a PWM timer.
 * - 1. check parameters.
 * - 2. reset a PWM timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_resetTimer(unsigned char chNo)
{
	// parameter check
	if(chNo >= T16E_CH_MAX) {
		return	T16E_INVALID_PARAMETER;
	}

	// PWM timer reset
	RegModify(R16_T16E_CTL(chNo), T16ERST, (T16ERST * T16E_RESET));

	return	T16E_SUCCESS;
}

/**
 * @brief	set a PWM timer run / stop.
 * - 1. check parameters.
 * - 2. set a PWM timer run/stop.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	selectBit; unsigned short: 		select run / stop
 * -		T16E_TIMER_STOP : PWM timer stop
 * -		T16E_TIMER_RUN  : PWM timer run
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_setTimerRun(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		(selectBit > T16E_TIMER_RUN) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// PWM timer run / stop
	RegModify(R16_T16E_CTL(chNo), T16ERUN, (selectBit << T16ERUN_SFT));

	return	T16E_SUCCESS;
}

/**
 * @brief	set a timer input clock.
 * - 1. check parameters.
 * - 2. set a timer input clock.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	selectBit; unsigned short:		select timer input clock
 * -		T16E_CNTCLK_1     : PCLK 1/1
 * -		T16E_CNTCLK_2     : PCLK 1/2
 * -		T16E_CNTCLK_4     : PCLK 1/4
 * -		T16E_CNTCLK_8     : PCLK 1/8
 * -		T16E_CNTCLK_16    : PCLK 1/16
 * -		T16E_CNTCLK_32    : PCLK 1/32
 * -		T16E_CNTCLK_64    : PCLK 1/64
 * -		T16E_CNTCLK_128   : PCLK 1/128
 * -		T16E_CNTCLK_256   : PCLK 1/256
 * -		T16E_CNTCLK_512   : PCLK 1/512
 * -		T16E_CNTCLK_1024  : PCLK 1/1024
 * -		T16E_CNTCLK_2048  : PCLK 1/2048
 * -		T16E_CNTCLK_4096  : PCLK 1/4096
 * -		T16E_CNTCLK_8192  : PCLK 1/8192
 * -		T16E_CNTCLK_16384 : PCLK 1/16384
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_setInputClock(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		(selectBit >= T16E_CNTCLK_MAX) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// clock stop
	T16E_setTimerRun(chNo, T16E_TIMER_STOP);

	// set count clock
	RegWrite(R16_T16E_CLK(chNo), selectBit);

	return	T16E_SUCCESS;
}

/**
 * @brief	PWM timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set a compare A/B match interrupt level.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	intLevel; unsigned char:		select compare A/B match interrupt level
 * -		T16E_INT_LEVEL0 : level0
 * -		T16E_INT_LEVEL1 : level1
 * -		T16E_INT_LEVEL2 : level2
 * -		T16E_INT_LEVEL3 : level3
 * -		T16E_INT_LEVEL4 : level4
 * -		T16E_INT_LEVEL5 : level5
 * -		T16E_INT_LEVEL6 : level6
 * -		T16E_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_initInt(unsigned char chNo, unsigned char intLevel)
{
	// parameter check
	if( (chNo >= T16E_CH_MAX) ||
		(intLevel >= T16E_INT_LEVELMAX) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(T16E_INTLVL_REG[chNo], intLevel);

	return	T16E_SUCCESS;
}

/**
 * @brief	PWM timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set a compare A/B match interrupt enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	selectBit; unsigned short:		select compare A/B match interrupt enable / disable
 * -		T16E_CMPBINT_DISABLE : compare B match interrupt disable
 * -		T16E_CMPBINT_ENABLE  : compare B match interrupt enable
 * -		T16E_CMPAINT_DISABLE : compare A match interrupt disable
 * -		T16E_CMPAINT_ENABLE  : compare A match interrupt enable
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_controlInt(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		((selectBit & ~T16E_INT_MASK) != 0) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// reset interrupt flag
	T16E_resetIntFlag(chNo, selectBit);

	// set interrupt enable
	RegModify(R16_T16E_IMSK(chNo), T16E_INT_MASK, selectBit);

	return	T16E_SUCCESS;
}

/**
 * @brief	PWM timer interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	selectBit; unsigned short:		reset interrupt status
 * -		T16E_BINT_RESET : compare B match interrupt status reset
 * -		T16E_AINT_RESET : compare A match interrupt status reset
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_resetIntFlag(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16E_CH_MAX) ||
		((selectBit & ~(CBIF | CAIF)) != 0) ) {
			return	T16E_INVALID_PARAMETER;
	}

	// reset interrupt flag
	RegWrite(R16_T16E_IFLG(chNo), selectBit);

	return	T16E_SUCCESS;
}

/**
 * @brief	PWM timer interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16E_CH0 : channel 0
 * -		T16E_CH1 : channel 1
 * @param	*flag; unsigned short:			check interrupt flag
 * -		T16E_BINT_NOTOCCURRED : compare B match interrupt is not occurred
 * -		T16E_BINT_OCCURRED    : compare B match interrupt is occurred
 * -		T16E_AINT_NOTOCCURRED : compare A match interrupt is not occurred
 * -		T16E_AINT_OCCURRED    : compare A match interrupt is occurred
 * @return	short:							end status
 * -		T16E_SUCCESS : success
 * -		T16E_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16E_checkIntFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= T16E_CH_MAX) {
		return	T16E_INVALID_PARAMETER;
	}

	// get interrupt flag
	*flag = (RegRead(R16_T16E_IFLG(chNo)) & (CBIF | CAIF)) >> CAIF_SFT;

	return	T16E_SUCCESS;
}

#endif	// end of PE_T16E
