/**
 * @file	t8f.c
 * @brief	8-bit timer driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_T8F
#include <t8f_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short T8F_init(unsigned char chNo, T8F_INIT_PARAM *param);
short T8F_setInputClock(unsigned char chNo, unsigned short selectBit);
short T8F_setReloadData(unsigned char chNo, unsigned char data);
short T8F_getCounterData(unsigned char chNo, unsigned char *data);
short T8F_setTimerMode(unsigned char chNo, unsigned short tfmd, unsigned short trmd);
short T8F_resetTimer(unsigned char chNo);
short T8F_setTimerRun(unsigned char chNo, unsigned short selectBit);
short T8F_initInt(unsigned char intLevel);
short T8F_controlInt(unsigned char chNo, unsigned short selectBit);
short T8F_resetIntFlag(unsigned char chNo);
short T8F_checkIntFlag(unsigned char chNo, unsigned short *flag);


/**
 * @brief	8-bit timer initialize.
 * - 1. check parameters.
 * - 2. stop a 8-bit timer.
 * - 3. set a count mode.
 * - 4. set a count clock.
 * - 5. set a reload count.
 * - 6. set a fine mode.
 * - 7. reset a counter.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	*param; T8F_INIT_PARAM: 		initialize parameter
 * -		tfmd; unsigned short:			select fine mode
 * 	-			refer to T8F_setTimerMode
 * -		trmd; unsigned short:			select count mode
 * 	-			T8F_TIMMODE_REPEAT  : repeat mode
 * 	-			T8F_TIMMODE_ONESHOT : one shot mode
 * -		selectClk; unsigned short:		select count clock
 * 	-			refer to T8F_setInputClock
 * -		data; unsigned short:			counter initial value
 * 	-			from 0x00 to 0xFF
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_init(unsigned char chNo, T8F_INIT_PARAM *param)
{
	// parameter check
	if(	(chNo >= T8F_CH_MAX) ||
		(param->tfmd >= T8F_TIMMODE_MAX) ||
		(param->trmd > T8F_TIMMODE_ONESHOT) ||
		(param->selectClk >= T8F_CNTCLK_MAX) ) {
			return	T8F_INVALID_PARAMETER;
	}
	
	// 8-bit timer stop & mode set
	T8F_setTimerMode(chNo, param->tfmd, param->trmd);
	
	// 8-bit timer input clock set
	T8F_setInputClock(chNo, param->selectClk);
	
	// 8-bit timer reload data set
	T8F_setReloadData(chNo, param->data);
	
	// 8-bit timer reset
	T8F_resetTimer(chNo);
	
	return	T8F_SUCCESS;
}

/**
 * @brief	set a timer input clock.
 * - 1. check parameters.
 * - 2. set a timer input clock.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	selectBit; unsigned short: 		select timer input clock
 * -		T8F_CNTCLK_1     : PCLK 1/1
 * -		T8F_CNTCLK_2     : PCLK 1/2
 * -		T8F_CNTCLK_4     : PCLK 1/4
 * -		T8F_CNTCLK_8     : PCLK 1/8
 * -		T8F_CNTCLK_16    : PCLK 1/16
 * -		T8F_CNTCLK_32    : PCLK 1/32
 * -		T8F_CNTCLK_64    : PCLK 1/64
 * -		T8F_CNTCLK_128   : PCLK 1/128
 * -		T8F_CNTCLK_256   : PCLK 1/256
 * -		T8F_CNTCLK_512   : PCLK 1/512
 * -		T8F_CNTCLK_1024  : PCLK 1/1024
 * -		T8F_CNTCLK_2048  : PCLK 1/2048
 * -		T8F_CNTCLK_4096  : PCLK 1/4096
 * -		T8F_CNTCLK_8192  : PCLK 1/8192
 * -		T8F_CNTCLK_16384 : PCLK 1/16384
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_setInputClock(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T8F_CH_MAX) ||
		(selectBit >= T8F_CNTCLK_MAX) ) {
			return	T8F_INVALID_PARAMETER;
	}
	
	// clock stop
	T8F_setTimerRun(chNo, T8F_TIMER_STOP);
	
	// set count clock
	RegWrite(R16_T8F_CLK(chNo), selectBit);
	
	return	T8F_SUCCESS;
}

/**
 * @brief	set a 8-bit timer reload data.
 * - 1. check parameters.
 * - 2. set a counter initial value of 8-bit timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	data; unsigned char:			counter initial value
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_setReloadData(unsigned char chNo, unsigned char data)
{
	// parameter check
	if(chNo >= T8F_CH_MAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// set counter value
	RegWrite(R16_T8F_TR(chNo), (unsigned short)data);
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	get a counter data.
 * - 1. check parameters.
 * - 2. get a counter data from register and set the data to "data".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	*data; unsigned char:			counter data
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_getCounterData(unsigned char chNo, unsigned char *data)
{
	// parameter check
	if(chNo >= T8F_CH_MAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// get counter data
	*data = (unsigned char)RegRead(R16_T8F_TC(chNo));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	set a 8-bit timer mode.
 * - 1. check parameters.
 * - 2. set a 8-bit timer mode.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	tfmd; unsigned short: 			setup fine mode
 * -		T8F_TIMMODE_PATTERN0 : pattern 0
 * -		T8F_TIMMODE_PATTERN1 : pattern 1
 * -		T8F_TIMMODE_PATTERN2 : pattern 2
 * -		T8F_TIMMODE_PATTERN3 : pattern 3
 * -		T8F_TIMMODE_PATTERN4 : pattern 4
 * -		T8F_TIMMODE_PATTERN5 : pattern 5
 * -		T8F_TIMMODE_PATTERN6 : pattern 6
 * -		T8F_TIMMODE_PATTERN7 : pattern 7
 * -		T8F_TIMMODE_PATTERN8 : pattern 8
 * -		T8F_TIMMODE_PATTERN9 : pattern 9
 * -		T8F_TIMMODE_PATTERNA : pattern A
 * -		T8F_TIMMODE_PATTERNB : pattern B
 * -		T8F_TIMMODE_PATTERNC : pattern C
 * -		T8F_TIMMODE_PATTERND : pattern D
 * -		T8F_TIMMODE_PATTERNE : pattern E
 * -		T8F_TIMMODE_PATTERNF : pattern F
 * @param	trmd; unsigned short: 			select count mode
 * -		T8F_TIMMODE_REPEAT  : repeat mode
 * -		T8F_TIMMODE_ONESHOT : one shot mode
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_setTimerMode(unsigned char chNo, unsigned short tfmd, unsigned short trmd)
{
	// parameter check
	if( (chNo >= T8F_CH_MAX) ||
		(tfmd >= T8F_TIMMODE_MAX) ||
		(trmd > T8F_TIMMODE_ONESHOT) ) {
			return	T8F_INVALID_PARAMETER;
	}
	
	// clock stop
	T8F_setTimerRun(chNo, T8F_TIMER_STOP);
	
	// set timer mode
	RegModify(R16_T8F_CTL(chNo), (T8TFMD |T8TRMD), ((tfmd << T8TFMD_SFT) | (trmd << T8TRMD_SFT)));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	reset a 8-bit timer.
 * - 1. check parameters.
 * - 2. reset a 8-bit timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_resetTimer(unsigned char chNo)
{
	// parameter check
	if(chNo >= T8F_CH_MAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// 8-bti timer reset
	RegModify(R16_T8F_CTL(chNo), T8PRESER, (T8PRESER * T8F_RESET));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	set a 8-bit timer run / stop.
 * - 1. check parameters.
 * - 2. set a 8-bit timer run/stop.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	selectBit; unsigned short: 		select run / stop
 * -		T8F_TIMER_STOP : 8-bit timer stop
 * -		T8F_TIMER_RUN  : 8-bit timer run
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_setTimerRun(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T8F_CH_MAX) ||
		(selectBit > T8F_TIMER_RUN) ) {
			return	T8F_INVALID_PARAMETER;
	}
	
	// 8-bit timer run / stop
	RegModify(R16_T8F_CTL(chNo), T8PRUN, (selectBit << T8PRUN_SFT));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	8-bit timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		T8F_INT_LEVEL0 : level0
 * -		T8F_INT_LEVEL1 : level1
 * -		T8F_INT_LEVEL2 : level2
 * -		T8F_INT_LEVEL3 : level3
 * -		T8F_INT_LEVEL4 : level4
 * -		T8F_INT_LEVEL5 : level5
 * -		T8F_INT_LEVEL6 : level6
 * -		T8F_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= T8F_INT_LEVELMAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(T8F_INTLVL, intLevel);
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	8-bit timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		T8F_INT_DISABLE : interrupt disable
 * -		T8F_INT_ENABLE  : interrupt enable
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_controlInt(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T8F_CH_MAX) ||
		(selectBit > T8F_INT_ENABLE) ) {
			return	T8F_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	T8F_resetIntFlag(chNo);
	
	// set interrupt enable
	RegModify(R16_T8F_INT(chNo), T8IE, (selectBit << T8IE_SFT));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	8-bit timer interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_resetIntFlag(unsigned char chNo)
{
	// parameter check
	if(chNo >= T8F_CH_MAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegModify(R16_T8F_INT(chNo), T8IF, (T8IF * T8F_RESET));
	
	return	T8F_SUCCESS;
}

/**  
 * @brief	8-bit timer interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 * 
 * @param	chNo; unsigned char:			channel number
 * -		T8F_CH0 : channel 0
 * -		T8F_CH1 : channel 1
 * @param	*flag; unsigned short:			check interrupt flag
 * -		T8F_TIMERINT_NOTOCCURRED : interrupt is not occurred
 * -		T8F_TIMERINT_OCCURRED    : interrupt is occurred
 * @return	short:							end status
 * -		T8F_SUCCESS : success
 * -		T8F_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8F_checkIntFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= T8F_CH_MAX) {
		return	T8F_INVALID_PARAMETER;
	}
	
	// get interrupt flag
	*flag = (RegRead(R16_T8F_INT(chNo)) & T8IF) >> T8IF_SFT;
	
	return	T8F_SUCCESS;
}

#endif	// end of PE_T8F
